from __future__ import print_function

__author__ = "Pierre Barbier de Reuille <pierre.barbierdereuille@gmail.com>"

from ..utils import namedtuple
from .. import loader
from path import path
import os

_fields = ['name', 'description', 'invert', 'Dfun', '__call__']

residuals = None

Residual = namedtuple('Residual', ['fct', 'name', 'description', 'invert', 'Dfun', '__call__'])


def find_functions(module):
    """
    Find and all the residual functions defined in the given module
    """
    content = dir(module)
    result = {}
    for c in content:
        obj = getattr(module, c)
        try:
            if isinstance(obj, type):
                obj = obj()
            for attr in _fields:
                if not hasattr(obj, attr):
                    break
            else:
                result[obj.name] = obj
        except Exception as ex:  # Silently ignore any exception
            print("Error: '{}'".format(ex))
            pass
    return result


def load():
    """
    Load and register all the residual functions available.

    It will be looking in the current folder, but also in the "residuals"
    subfolders of the paths defined in the PYQTFIT_PATH environment
    variable.
    """
    global residuals
    residuals = loader.load(find_functions)
    extra_path = os.environ.get("PYQTFIT_PATH", "").split(":")
    for ep in extra_path:
        ep = path(ep)
        if ep and (ep / "residuals").exists():
            residuals.update(loader.load(find_functions, ep / "residuals"))
    return residuals

load()


def get(name):
    """
    Return the description of the residuals names 'name'

    Parameters
    ----------
    name: str
        Name of the residuals

    Returns
    -------
    fct: callable object
        Callable object that will compute the residuals. The callable has the following attributes:
        fct: callable
            Function computing the residuals
        name: str
            Name of the residuals type
        description: str
            Formula of the residual
        invert: callable
            Function to add residuals to a data set
    """
    return residuals.get(name, None)


def names():
    """
    List the names of available residuals
    """
    return residuals.keys()



from __future__ import print_function

__author__ = "Pierre Barbier de Reuille <pierre.barbierdereuille@gmail.com>"

from ..utils import namedtuple
from .. import loader
import os
from path import path

_fields = ['name', 'description', 'args', 'init_args', 'Dfun', '__call__']

functions = None

Function = namedtuple('Function', _fields)


def find_functions(module):
    """
    Find and register the functions defined in the given module
    """
    content = dir(module)
    result = {}
    for c in content:
        obj = getattr(module, c)
        try:
            if isinstance(obj, type):
                obj = obj()
            for attr in _fields:
                if not hasattr(obj, attr):
                    break
            else:
                result[obj.name] = obj
        except Exception as ex:  # Silently ignore any exception
            print("Error: '{}'".format(ex))
            pass
    return result


def load():
    """
    Find and register the all the functions available.

    It will be looking in the current folder, but also in the "functions"
    subfolders of the paths specified in the PYQTFIT_PATH environment variable.
    """
    global functions
    functions = loader.load(find_functions)
    extra_path = os.environ.get("PYQTFIT_PATH", "").split(":")
    for ep in extra_path:
        ep = path(ep)
        if ep and (ep / "functions").exists():
            functions.update(loader.load(find_functions, ep / "functions"))
    return functions

load()


def get(name):
    """
    Get a tuple representing the function or None if not found

    Parameters
    ----------
    name: str
        Name of the function to find

    Returns
    -------
    fct: namedtuple
        Function of interest. The named tuple also has the following fields:
        fct: callable
            The function itself
        name: string
            Name of the function
        description: string
            Description (i.e. formula) of the function
        args: tuple of string
            List of argument names
        init_args: callable
            Function used to estimate initial parameters from dataset
        Dfun: callable
            Jacobian of the function (in column, so call leastsq with col_deriv=1)
    """
    f = functions.get(name, None)
    #print("Getting function '{}'".format(name))
    #if f is not None:
        #print("  Dfun = {}".format(f.Dfun))
    return f


def names():
    """
    Return the list of names for the functions
    """
    return functions.keys()



from __future__ import print_function, absolute_import
import inspect
from path import path
import imp
import sys
import re

bad_chars = re.compile(u'\W')

sys_modules = "*.so"
if sys.platform == 'win32' or sys.platform == 'cygwin':
    sys_modules = "*.dll"
elif sys.platform == 'darwin':
    sys_modules = "*.dylib"


def load(find_functions, search_path=None):
    """
    Load the modules in the search_path.
    If search_path is None, then load modules in the same folder as the function looking for them.
    """
    caller_module = inspect.getmodule(inspect.stack()[1][0])
    system_files = [caller_module.__file__]
    module_path = path(caller_module.__file__).abspath().dirname()
    sys_files = set()
    for f in system_files:
        if f.endswith(".pyo") or f.endswith(".pyc"):
            f = f[:-3] + "py"
        sys_files.add(path(f).abspath())
    if search_path is None:
        search_path = module_path
    else:
        search_path = path(search_path).abspath()
    fcts = {}
# Search for python, cython and modules
    for f in (search_path.files("*.py") +
              search_path.files("*.pyx") +
              search_path.files(sys_modules)):
        if f not in sys_files:
            module_name = f.namebase
            pack_name = '%s.%s_%s' % (caller_module.__name__,
                                      bad_chars.sub('_', module_path),
                                      module_name)
            try:
                mod_desc = imp.find_module(module_name, [search_path])
                mod = imp.load_module(pack_name, *mod_desc)
                fcts.update(find_functions(mod))
            except ImportError as ex:
                print("Warning, cannot import module '{0}' from {1}: {2}"
                      .format(module_name, caller_module.__name__, ex), file=sys.stderr)
    return fcts



cdef extern from "math.h" nogil:
    int isfinite(double v)
    long double powl(long double, long double)
    long double expl(long double)
    long double logl(long double)
    long double sqrtl(long double)
    long double lgammal(long double)
    double erf(double)
    double fabs(double)



from __future__ import division, absolute_import, print_function

from .. import kde
import numpy as np
from numpy import newaxis
from numpy.random import randn
from scipy import stats, integrate
from ..compat import irange


class TestBandwidth(object):

    @classmethod
    def setUpClass(cls):
        cls.ratios = np.array([1., 2., 5.])
        d = randn(500)
        cls.vs = cls.ratios[:, newaxis]*np.array([d, d, d])
        cls.ss = np.var(cls.vs, axis=1)

    def variance_methods(self, m):
        bws = np.array([m(v) for v in self.vs])
        assert bws.shape == (3, 1, 1)
        rati = bws[:, 0, 0] / self.ss
        assert sum((rati - rati[0])**2) < 1e-6
        rati = bws[:, 0, 0] / bws[0, 0, 0]
        assert sum((rati - self.ratios**2)**2) < 1e-6

    def test_variance_methods(self):
        yield self.variance_methods, kde.silverman_bandwidth
        yield self.variance_methods, kde.scotts_bandwidth

    def test_botev(self):
        class FakeModel(object):
            lower = -np.inf
            upper = np.inf
        bws = np.array([kde.botev_bandwidth()(v, model=FakeModel()) for v in self.vs])
        assert bws.shape == (3,)
        rati = bws**2 / self.ss
        assert sum((rati - rati[0])**2) < 1e-6
        rati = bws / bws[0]
        assert sum((rati - self.ratios)**2) < 1e-6


class TestUnboundedKDE1D(object):
    @classmethod
    def setUpClass(cls):
        cls.dist = stats.norm(0, 1)
        cls.sizes = np.r_[1000:5000:5j]
        cls.vs = [cls.dist.rvs(s) for s in cls.sizes]
        cls.args = {}
        cls.grid_accuracy = 1e-6
        cls.accuracy = 1e-3

    def createKDE(self, data, **args):
        return kde.KDE1D(data, **args)

    #def test_converge(self):
        #xs = np.r_[-3:3:512j]
        #ys = self.dist.pdf(xs)
        #ks = [ self.createKDE(v, **self.args) for v in self.vs ]

    def is_normed(self, i):
        k = self.createKDE(self.vs[i], **self.args)
        xs, ys = k._grid_eval(2048)
        tot = integrate.simps(ys, xs)
        assert abs(tot - 1) < self.accuracy, "Error, {} should be close to 1".format(tot)

    def is_grid_normed(self, i):
        k = self.createKDE(self.vs[i], **self.args)
        xs, ys = k.grid(2048)
        tot = integrate.simps(ys, xs)
        assert abs(tot - 1) < self.grid_accuracy, "Error, {} should be close to 1".format(tot)

    def test_normed(self):
        for i in irange(len(self.sizes)):
            yield self.is_normed, i

    def test_grid_normed(self):
        for i in irange(len(self.sizes)):
            yield self.is_grid_normed, i

    def is_ws_normed(self, i):
        ws = np.r_[1:2:self.sizes[i]*1j]
        k = self.createKDE(self.vs[i], weights=ws, **self.args)
        xs, ys = k._grid_eval(2048)
        tot = integrate.simps(ys, xs)
        assert abs(tot - 1) < self.accuracy, "Error, {} should be close to 1".format(tot)

    def is_ws_grid_normed(self, i):
        ws = np.r_[1:2:self.sizes[i]*1j]
        k = self.createKDE(self.vs[i], weights=ws, **self.args)
        xs, ys = k.grid(2048)
        tot = integrate.simps(ys, xs)
        assert abs(tot - 1) < self.grid_accuracy, "Error, {} should be close to 1".format(tot)

    def test_ws_normed(self):
        for i in irange(len(self.sizes)):
            yield self.is_ws_normed, i

    def test_ws_grid_normed(self):
        for i in irange(len(self.sizes)):
            yield self.is_ws_grid_normed, i

    def is_ls_normed(self, i):
        ws = np.r_[1:2:self.sizes[i]*1j]
        k = self.createKDE(self.vs[i], lambdas=ws, **self.args)
        xs, ys = k._grid_eval(2048)
        tot = integrate.simps(ys, xs)
        assert abs(tot - 1) < self.accuracy, "Error, {} should be close to 1".format(tot)

    def test_ls_normed(self):
        for i in irange(len(self.sizes)):
            yield self.is_ls_normed, i


class TestReflexionKDE1D(TestUnboundedKDE1D):
    @classmethod
    def setUpClass(cls):
        cls.dist = stats.norm(0, 1)
        cls.sizes = np.r_[1000:5000:5j]
        cls.vs = [cls.dist.rvs(s) for s in cls.sizes]
        cls.args = dict(lower=-5, upper=5, method='reflexion')
        cls.grid_accuracy = 1e-5
        cls.accuracy = 1e-3


class TestCyclicKDE1D(TestUnboundedKDE1D):
    @classmethod
    def setUpClass(cls):
        cls.dist = stats.norm(0, 1)
        cls.sizes = np.r_[1000:5000:5j]
        cls.vs = [cls.dist.rvs(s) for s in cls.sizes]
        cls.args = dict(lower=-5, upper=5, method='cyclic')
        cls.grid_accuracy = 1e-5
        cls.accuracy = 1e-3


class TestRenormKDE1D(TestUnboundedKDE1D):
    @classmethod
    def setUpClass(cls):
        cls.dist = stats.norm(0, 1)
        cls.sizes = np.r_[1000:5000:5j]
        cls.vs = [cls.dist.rvs(s) for s in cls.sizes]
        cls.args = dict(lower=-5, upper=5, method='renormalization')
        cls.grid_accuracy = 1e-4
        cls.accuracy = 1e-3


class TestLCKDE1D(TestUnboundedKDE1D):
    @classmethod
    def setUpClass(cls):
        cls.dist = stats.norm(0, 1)
        cls.sizes = np.r_[1000:5000:5j]
        cls.vs = [cls.dist.rvs(s) for s in cls.sizes]
        cls.args = dict(lower=-5, upper=5, method='linear_combination')
        cls.grid_accuracy = 1e-4
        cls.accuracy = 1e-3



"""
:Author: Pierre Barbier de Reuille <pierre.barbierdereuille@gmail.com>

This package is designed to perform n-D least-square fitting of user-defined
functions. It also provides a GUI that can use pre-defined fitting methods.
"""

from __future__ import absolute_import, print_function

__all__ = ['bootstrap', 'plot_fit',
           'curve_fitting', 'kernel_smoothing',
           'functions', 'residuals', 'CurveFitting']

from . import functions
from . import residuals
from .curve_fitting import CurveFitting
from path import path

with (path(__file__).dirname() / 'version.txt').open() as f:
    __version__ = f.read().strip()



from __future__ import division, absolute_import, print_function

from .. import utils
from nose.tools import raises


class TestNamedTuple(object):
    def test_normal(self):
        nt = utils.namedtuple('nt', 'x y z')
        n = nt(1, 2, 3)
        assert hasattr(n, 'x')
        assert n.x == 1
        assert hasattr(n, 'y')
        assert n.y == 2
        assert hasattr(n, 'z')
        assert n.z == 3

    def test_replace(self):
        nt = utils.namedtuple('nt', 'x y z')
        n = nt(1, 2, 3)
        n1 = n._replace(x=10)
        assert n1.x == 10
        assert n1.y == 2
        assert n1.z == 3

    def test_extended(self):
        nt = utils.namedtuple('nt', 'x __call__ __tested__')
        n = nt(1, lambda x: x, 3)
        assert hasattr(n, 'x')
        assert hasattr(n, '__call__')
        assert hasattr(n, '__tested__')
        assert n(True)

    @raises(ValueError)
    def test_badname(self):
        utils.namedtuple('nt', 'x y _test1')

    @raises(ValueError)
    def reserved(self, n):
        utils.namedtuple('nt', n)

    def test_reserved_names(self):
        for n in ['__init__', '__slots__', '__new__', '__repr__', '__getnewargs__']:
            yield self.reserved, n

    @raises(ValueError)
    def test_keyword(self):
        utils.namedtuple('nt', 'is')

    @raises(ValueError)
    def test_duplicate(self):
        utils.namedtuple('nt', 'x y x')

    @raises(ValueError)
    def test_number(self):
        utils.namedtuple('nt', 'x 12 x')

    @raises(ValueError)
    def test_symbol(self):
        utils.namedtuple('nt', 'x y-t x')

import numpy as np


def square(vals):
    return vals ** 2


class TestJacobian(object):
    def test_call(self):
        res = utils.approx_jacobian(np.array([0, 0]), square, 1)
        np.testing.assert_array_equal(res, np.eye(2, dtype=res.dtype))

    def test_call2(self):
        res = utils.approx_jacobian(np.array([1, 1]), square, 1e-6)
        np.testing.assert_array_almost_equal(res, 2 * np.eye(2, dtype=res.dtype), 6)

    def call_prec(self, p, dt=float):
        res = utils.approx_jacobian(np.array([1, 1], dtype=dt), square, 10 ** (-p))
        np.testing.assert_array_almost_equal(res, 2 * np.eye(2, dtype=res.dtype), p)

    def test_call_prec(self):
        for p in (3, 4, 5, 6, 8):  # Above 8, we hit precision limit of floating point
            yield self.call_prec, p

    def test_call_high_prec(self):
        for p in (5, 8, 9):  # Above 9, we hit precision limit of floating point
            yield self.call_prec, p, utils.large_float



"""
:Author: Pierre Barbier de Reuille <pierre.barbierdereuille@gmail.com>

This modules implement functions to test and plot parametric regression.
"""

from __future__ import division, print_function, absolute_import
from numpy import argsort, std, abs, sqrt, arange, pi, c_
from pylab import figure, title, legend, plot, xlabel, ylabel, subplot, ylim, hist, suptitle, gca
from .compat import izip
from itertools import chain
from scipy.special import erfinv, gamma
from scipy import stats
from .kernel_smoothing import LocalLinearKernel1D
from .compat import unicode_csv_writer as csv_writer
from collections import namedtuple

smoothing = LocalLinearKernel1D

import sys
if sys.version_info >= (3,):
    CSV_WRITE_FLAGS = "wt"
else:
    CSV_WRITE_FLAGS = "wb"


def plot_dist_residuals(res):
    """
    Plot the distribution of the residuals.

    :returns: the handle toward the histogram and the plot of the fitted normal distribution
    """
    ph = hist(res, normed=True)
    xr = arange(res.min(), res.max(), (res.max() - res.min()) / 1024)
    yr = stats.norm(0, res.std()).pdf(xr)
    pn = plot(xr, yr, 'r--')
    xlabel('Residuals')
    ylabel('Frequency')
    title('Distributions of the residuals')
    return ph, pn


def plot_residuals(xname, xdata, res_desc, res):
    """
    Plot the residuals against the X axis

    :param str     xname:    Name of the X axis
    :param ndarray xdata:    1D array with the X data
    :param str     res_desc: Name of the Y axis
    :param ndarray res: 1D   array with the residuals

    The shapes of ``xdata`` and ``res`` must be the same

    :returns: The handles of the the plots of the residuals and of the smoothed residuals.
    """
    p_res = plot(xdata, res, '+', label='residuals')[0]
    plot([xdata.min(), xdata.max()], [0, 0], 'r--')
    av = LocalLinearKernel1D(xdata, res)
    xr = arange(xdata.min(), xdata.max(), (xdata.max() - xdata.min()) / 1024)
    rr = av(xr)
    p_smooth = plot(xr, rr, 'g', label='smoothed residuals')
    xlabel(xname)
    ylabel("Residuals")
    ymin, ymax = ylim()
    ymax = max(ymax, -ymin)
    ylim(-ymax, ymax)
    title("Residuals (%s) vs. fitted" % (res_desc,))
    return p_res, p_smooth


def scaled_location_plot(yname, yopt, scaled_res):
    """
    Plot the scaled location, given the dependant values and scaled residuals.

    :param str     yname:      Name of the Y axis
    :param ndarray yopt:       Estimated values
    :param ndarray scaled_res: Scaled residuals

    :returns: the handles for the data and the smoothed curve
    """

    scr = sqrt(abs(scaled_res))
    p_scaled = plot(yopt, scr, '+')[0]
    av = LocalLinearKernel1D(yopt, scr)
    xr = arange(yopt.min(), yopt.max(), (yopt.max() - yopt.min()) / 1024)
    rr = av(xr)
    p_smooth = plot(xr, rr, 'g')[0]
    expected_mean = 2 ** (1 / 4) * gamma(3 / 4) / sqrt(pi)
    plot([yopt.min(), yopt.max()], [expected_mean, expected_mean], 'r--')
    title('Scale-location')
    xlabel(yname)
    ylabel('$|$Normalized residuals$|^{1/2}$')
    gca().set_yticks([0, 1, 2])
    return [p_scaled, p_smooth]


def qqplot(scaled_res, normq):
    """
    Draw a Q-Q Plot from the sorted, scaled residuals (i.e. residuals sorted
    and normalized by their standard deviation)

    :param ndarray scaled_res: Scaled residuals
    :param ndarray normq:      Expected value for each scaled residual, based on its quantile.

    :returns: handle to the data plot
    """
    qqp = []
    qqp += plot(normq, scaled_res, '+')
    qqp += plot(normq, normq, 'r--')
    xlabel('Theoretical quantiles')
    ylabel('Normalized residuals')
    title('Normal Q-Q plot')
    return qqp

ResultStruct = namedtuple('ResultStruct', "fct fct_desc param_names xdata ydata xname yname "
                          "res_name residuals popt res yopts eval_points interpolation "
                          "sorted_yopts scaled_res normq CI CIs CIresults")


def fit_evaluation(fit, xdata, ydata, eval_points=None,
                   CI=(), CIresults = None, xname="X", yname="Y",
                   fct_desc=None, param_names=(), residuals=None, res_name='Standard'):
    """
    This function takes the output of a curve fitting experiment and store all the relevant
    information for evaluating its success in the result.

    :type  fit: fitting object
    :param fit: object configured for the fitting

    :type  xdata: ndarray of shape (N,) or (k,N) for function with k prefictors
    :param xdata: The independent variable where the data is measured

    :type  ydata: ndarray
    :param ydata: The dependant data

    :type  eval_points: ndarray or None
    :param eval_points: Contain the list of points on which the result must be expressed. It is
        used both for plotting and for the bootstrapping.

    :type  CI: tuple of int
    :param CI: List of confidence intervals to calculate. If empty, none are calculated.

    :type  xname: string
    :param xname: Name of the X axis

    :type  yname: string
    :param yname: Name of the Y axis

    :type  fct_desc: string
    :param fct_desc: Formula of the function

    :type  param_names: tuple of strings
    :param param_names: Name of the various parameters

    :type  residuals: callable
    :param residuals: Residual function

    :type  res_desc: string
    :param res_desc: Description of the residuals

    :rtype: :py:class:`ResultStruct`
    :returns: Data structure summarising the fitting and its evaluation
    """
    popt = fit.popt
    res = fit.res

    if CI:
        CIs = CIresults.CIs
    else:
        CIs = []

    yopts = fit(xdata)
    if eval_points is None:
        yvals = yopts
        eval_points = xdata
    else:
        yvals = fit(eval_points)

    scaled_res, res_IX, prob, normq = residual_measures(res)
    sorted_yopts = yopts[res_IX]

    result = {}
    result["fct"] = fit
    result["fct_desc"] = fct_desc
    result["param_names"] = param_names
    result["xdata"] = xdata
    result["ydata"] = ydata
    result["xname"] = xname
    result["yname"] = yname
    result["res_name"] = res_name
    result["residuals"] = residuals
    #result["args"] = fit.args
    result["popt"] = popt
    result["res"] = res
    result["yopts"] = yopts
    result["eval_points"] = eval_points
    result["interpolation"] = yvals
    result["sorted_yopts"] = sorted_yopts
    result["scaled_res"] = scaled_res
    result["normq"] = normq
    result["CI"] = CI
    result["CIs"] = CIs
    #result["CIparams"] = CIparams
    result["CIresults"] = CIresults
    #print("estimate jacobian = %s" % result["extra_output"][-1]["est_jacobian"])
    return ResultStruct(**result)

ResidualMeasures = namedtuple("ResidualMeasures", "scaled_res res_IX prob normq")


def residual_measures(res):
    """
    Compute quantities needed to evaluate the quality of the estimation, based solely
    on the residuals.

    :rtype: :py:class:`ResidualMeasures`
    :returns: the scaled residuals, their ordering, the theoretical quantile for each residuals,
        and the expected value for each quantile.
    """
    IX = argsort(res)
    scaled_res = res[IX] / std(res)

    prob = (arange(len(scaled_res)) + 0.5) / len(scaled_res)
    normq = sqrt(2) * erfinv(2 * prob - 1)

    return ResidualMeasures(scaled_res, IX, prob, normq)

_restestfields = "res_figure residuals scaled_residuals qqplot dist_residuals"
ResTestResult = namedtuple("ResTestResult", _restestfields)
Plot1dResult = namedtuple("Plot1dResult", "figure estimate data CIs " + _restestfields)


def plot1d(result, loc=0, fig=None, res_fig=None):
    """
    Use matplotlib to display the result of a fit, and return the list of plots used

    :rtype: :py:class:`Plot1dResult`
    :returns: hangles to the various figures and plots
    """
    if fig is None:
        fig = figure()
    else:
        try:
            figure(fig)
        except TypeError:
            figure(fig.number)

    p_est = plot(result.eval_points, result.interpolation, label='estimated')[0]
    p_data = plot(result.xdata, result.ydata, '+', label='data')[0]
    p_CIs = []
    if result.CI:
        for p, (low, high) in izip(result.CI, result.CIs[0]):
            l = plot(result.eval_points, low, '--', label='%g%% CI' % (p,))[0]
            h = plot(result.eval_points, high, l.get_color() + '--')[0]
            p_CIs += [l, h]
    if result.param_names:
        param_strs = ", ".join("%s=%g" % (n, v) for n, v in izip(result.param_names, result.popt))
    else:
        param_strs = ", ".join("%g" % v for v in result.popt)
    param_strs = "$%s$" % (param_strs,)

    title("Estimated function %s with params %s" % (result.fct_desc, param_strs))

    xlabel(result.xname)
    ylabel(result.yname)
    legend(loc=loc)

    plots = {"figure": fig, "estimate": p_est, "data": p_data, "CIs": p_CIs}

    prt = plot_residual_tests(result.xdata, result.yopts, result.res,
                              "{0} with params {1}".format(result.fct_desc, param_strs),
                              result.xname, result.yname, result.res_name, result.sorted_yopts,
                              result.scaled_res,
                              result.normq, res_fig)

    plots.update(prt._asdict())

    return Plot1dResult(**plots)


def plot_residual_tests(xdata, yopts, res, fct_name, xname="X", yname='Y', res_name="residuals",
                        sorted_yopts=None, scaled_res=None, normq=None, fig=None):
    """
    Plot, in a single figure, all four residuals evaluation plots: :py:func:`plot_residuals`,
    :py:func:`plot_dist_residuals`, :py:func:`scaled_location_plot` and :py:func:`qqplot`.

    :param ndarray xdata:        Explaining variables
    :param ndarray yopt:         Optimized explained variables
    :param str     fct_name:     Name of the fitted function
    :param str     xname:        Name of the explaining variables
    :param str     yname:        Name of the dependant variables
    :param str     res_name:     Name of the residuals
    :param ndarray sorted_yopts: ``yopt``, sorted to match the scaled residuals
    :param ndarray scaled_res:   Scaled residuals
    :param ndarray normq:        Estimated value of the quantiles for a normal distribution

    :type  fig: handle or None
    :param fig: Handle of the figure to put the plots in, or None to create a new figure

    :rtype: :py:class:`ResTestResult`
    :returns: The handles to all the plots
    """
    if fig is None:
        fig = figure()
    else:
        try:
            figure(fig)
        except TypeError:
            figure(fig.number)

    subplot(2, 2, 1)
# First subplot is the residuals
    if len(xdata.shape) == 1 or xdata.shape[1] == 1:
        p_res = plot_residuals(xname, xdata.squeeze(), res_name, res)
    else:
        p_res = plot_residuals(yname, yopts, res_name, res)

    if scaled_res is None or sorted_yopts is None or normq is None:
        scaled_res, res_IX, _, normq = residual_measures(res)
        sorted_yopts = yopts[res_IX]

    subplot(2, 2, 2)
    p_scaled = scaled_location_plot(yname, sorted_yopts, scaled_res)

    subplot(2, 2, 3)
# Q-Q plot
    qqp = qqplot(scaled_res, normq)

    subplot(2, 2, 4)
# Distribution of residuals
    drp = plot_dist_residuals(res)

    suptitle("Residual Test for {}".format(fct_name))

    return ResTestResult(fig, p_res, p_scaled, qqp, drp)


def write1d(outfile, result, res_desc, CImethod):
    """
    Write the result of a fitting and its evaluation to a CSV file.

    :param str          outfile:  Name of the file to write to
    :param ResultStruct result:   Result of the fitting evaluation
        (e.g. output of :py:func:`fit_evaluation`)
    :param str          res_desc: Description of the residuals
        (in more details than just the name of the residuals)
    :param str          CImethod: Description of the confidence interval estimation method
    """
    with open(outfile, CSV_WRITE_FLAGS) as f:
        w = csv_writer(f)
        w.writerow(["Function", result.fct.fct.description])
        w.writerow(["Residuals", result.res_name, res_desc])
        w.writerow(["Parameter", "Value"])
        for pn, pv in izip(result.param_names, result.popt):
            w.writerow([pn, "%.20g" % pv])
        #TODO w.writerow(["Regression Evaluation"])
        w.writerow([])
        w.writerow(["Data"])
        w.writerow([result.xname, result.yname, result.fct_desc, "Residuals: %s" % result.res_name])
        w.writerows(c_[result.xdata, result.ydata, result.yopts, result.res])
        w.writerow([])
        w.writerow(['Model validation'])
        w.writerow([result.yname, 'Normalized residuals', 'Theoretical quantiles'])
        w.writerows(c_[result.sorted_yopts, result.scaled_res, result.normq])
        if result.eval_points is not result.xdata:
            w.writerow([])
            w.writerow(["Interpolated data"])
            w.writerow([result.xname, result.yname])
            w.writerows(c_[result.eval_points, result.interpolation])
        if result.CI:
            w.writerow([])
            w.writerow(["Confidence interval"])
            w.writerow(["Method", CImethod])
            head = ["Parameters"] + \
                list(chain(*[["%g%% - low" % v, "%g%% - high" % v] for v in result.CI]))
            w.writerow(head)
            #print(result.CIs[1])
            for cis in izip(result.param_names, *chain(*result.CIs[1])):
                cistr = [cis[0]] + ["%.20g" % v for v in cis[1:]]
                w.writerow(cistr)
            w.writerow([result.yname])
            head[0] = result.xname
            w.writerow(head)
            w.writerows(c_[tuple(chain([result.eval_points], *result.CIs[0]))])


# /home/barbier/prog/python/curve_fitting/test.csv



#!/usr/bin/python
import matplotlib
matplotlib.use('Qt4Agg')
from pyqt_fit import pyqt_fit1d
from PyQt4 import QtGui
import matplotlib

import sys
app = QtGui.QApplication(sys.argv)
matplotlib.interactive(True)
wnd = pyqt_fit1d.main()
sys.exit(app.exec_())




from __future__ import division, absolute_import, print_function
import numpy as np
from scipy.special import erf

from .cyth import HAS_CYTHON

kernels_imp = None


def usePython():
    """
    Force the use of the Python implementation of the kernels
    """
    global kernels_imp
    from .import _kernels_py
    kernels_imp = _kernels_py


def useCython():
    """
    Force the use of the Cython implementation of the kernels, if available
    """
    global kernels_imp
    if HAS_CYTHON:
        from . import _kernels
        kernels_imp = _kernels


if HAS_CYTHON:
    useCython()
else:
    usePython()
    import sys
    print("Warning, cannot import Cython kernel functions, "
          "pure python functions will be used instead", file=sys.stderr)

S2PI = np.sqrt(2 * np.pi)


S2 = np.sqrt(2)


class normal_kernel1d(object):
    """
    1D normal density kernel with extra integrals for 1D bounded kernel estimation.
    """

    def pdf(self, z, out=None):
        r"""
        Return the probability density of the function. The formula used is:

        .. math::

            \phi(z) = \frac{1}{\sqrt{2\pi}}e^{-\frac{x^2}{2}}

        :param ndarray xs: Array of any shape
        :returns: an array of shape identical to ``xs``
        """
        return kernels_imp.norm1d_pdf(z, out)

    def _pdf(self, z, out=None):
        """
        Full-python implementation of :py:func:`normal_kernel1d.pdf`
        """
        z = np.asarray(z)
        if out is None:
            out = np.empty(z.shape, dtype=z.dtype)
        np.multiply(z, z, out)
        out *= -0.5
        np.exp(out, out)
        out /= S2PI
        return out

    __call__ = pdf

    def fft(self, z, out=None):
        """
        Returns the FFT of the normal distribution
        """
        out = np.multiply(z, z, out)
        out *= -0.5
        np.exp(out, out)
        return out

    def dct(self, z, out=None):
        """
        Returns the DCT of the normal distribution
        """
        out = np.multiply(z, z, out)
        out *= -0.5
        np.exp(out, out)
        return out

    def cdf(self, z, out=None):
        r"""
        Cumulative density of probability. The formula used is:

        .. math::

            \text{cdf}(z) \triangleq \int_{-\infty}^z \phi(z)
                dz = \frac{1}{2}\text{erf}\left(\frac{z}{\sqrt{2}}\right) + \frac{1}{2}
        """
        return kernels_imp.norm1d_cdf(z, out)

    def _cdf(self, z, out=None):
        """
        Full-python implementation of :py:func:`normal_kernel1d.cdf`
        """
        z = np.asarray(z)
        if out is None:
            out = np.empty(z.shape, dtype=z.dtype)
        np.divide(z, S2, out)
        erf(out, out)
        out *= 0.5
        out += 0.5
        return out

    def pm1(self, z, out=None):
        r"""
        Partial moment of order 1:

        .. math::

            \text{pm1}(z) \triangleq \int_{-\infty}^z z\phi(z) dz
                = -\frac{1}{\sqrt{2\pi}}e^{-\frac{z^2}{2}}
        """
        return kernels_imp.norm1d_pm1(z, out)

    def _pm1(self, z, out=None):
        """
        Full-python implementation of :py:func:`normal_kernel1d.pm1`
        """
        z = np.asarray(z)
        if out is None:
            out = np.empty(z.shape, dtype=z.dtype)
        np.multiply(z, z, out)
        out *= -0.5
        np.exp(out, out)
        out /= -S2PI
        return out

    def pm2(self, z, out=None):
        r"""
        Partial moment of order 2:

        .. math::

            \text{pm2}(z) \triangleq \int_{-\infty}^z z^2\phi(z) dz
                = \frac{1}{2}\text{erf}\left(\frac{z}{2}\right) - \frac{z}{\sqrt{2\pi}}
                e^{-\frac{z^2}{2}} + \frac{1}{2}
        """
        return kernels_imp.norm1d_pm2(z, out)

    def _pm2(self, z, out=None):
        """
        Full-python implementation of :py:func:`normal_kernel1d.pm2`
        """
        z = np.asarray(z, dtype=float)
        if out is None:
            out = np.empty(z.shape)
        np.divide(z, S2, out)
        erf(out, out)
        out /= 2
        if z.shape:
            zz = np.isfinite(z)
            sz = z[zz]
            out[zz] -= sz * np.exp(-0.5 * sz * sz) / S2PI
        elif np.isfinite(z):
            out -= z * np.exp(-0.5 * z * z) / S2PI
        out += 0.5
        return out


class normal_kernel(object):
    """
    Returns a function-object for the PDF of a Normal kernel of variance
    identity and average 0 in dimension ``dim``.
    """

    def __new__(klass, dim):
        """
        The __new__ method will automatically select :py:class:`normal_kernel1d` if dim is 1.
        """
        if dim == 1:
            return normal_kernel1d()
        return object.__new__(klass, dim)

    def __init__(self, dim):
        self.factor = 1 / np.sqrt(2 * np.pi) ** dim

    def pdf(self, xs):
        """
        Return the probability density of the function.

        :param ndarray xs: Array of shape (D,N) where D is the dimension of the kernel
            and N the number of points.
        :returns: an array of shape (N,) with the density on each point of ``xs``
        """
        xs = np.atleast_2d(xs)
        return self.factor * np.exp(-0.5 * np.sum(xs * xs, axis=0))

    __call__ = pdf


class tricube(object):
    r"""
    Return the kernel corresponding to a tri-cube distribution, whose expression is.
    The tri-cube function is given by:

    .. math::

        f_r(x) = \left\{\begin{array}{ll}
                        \left(1-|x|^3\right)^3 & \text{, if } x \in [-1;1]\\
                                0 & \text{, otherwise}
                        \end{array}\right.

    As :math:`f_r` is not a probability and is not of variance 1, we use a normalized function:

    .. math::

        f(x) = a b f_r(ax)

        a = \sqrt{\frac{35}{243}}

        b = \frac{70}{81}

    """

    def pdf(self, z, out=None):
        return kernels_imp.tricube_pdf(z, out)

    __call__ = pdf

    def cdf(self, z, out=None):
        r"""
        CDF of the distribution:

        .. math::

            \text{cdf}(x) = \left\{\begin{array}{ll}
                \frac{1}{162} {\left(60 (ax)^{7} - 7 {\left(2 (ax)^{10} + 15 (ax)^{4}\right)}
                \mathrm{sgn}\left(ax\right) + 140 ax + 81\right)} & \text{, if}x\in[-1/a;1/a]\\
                0 & \text{, if} x < -1/a \\
                1 & \text{, if} x > 1/a
                \end{array}\right.
        """
        return kernels_imp.tricube_cdf(z, out)

    def pm1(self, z, out=None):
        r"""
        Partial moment of order 1:

        .. math::

            \text{pm1}(x) = \left\{\begin{array}{ll}
                \frac{7}{3564a} {\left(165 (ax)^{8} - 8 {\left(5 (ax)^{11} + 33 (ax)^{5}\right)}
                \mathrm{sgn}\left(ax\right) + 220 (ax)^{2} - 81\right)}
                & \text{, if} x\in [-1/a;1/a]\\
                0 & \text{, otherwise}
                \end{array}\right.
        """
        return kernels_imp.tricube_pm1(z, out)

    def pm2(self, z, out=None):
        r"""
        Partial moment of order 2:

        .. math::

            \text{pm2}(x) = \left\{\begin{array}{ll}
            \frac{35}{486a^2} {\left(4 (ax)^{9} + 4 (ax)^{3} - {\left((ax)^{12} + 6 (ax)^{6}\right)}
            \mathrm{sgn}\left(ax\right) + 1\right)} & \text{, if} x\in[-1/a;1/a] \\
            0 & \text{, if } x < -1/a \\
            1 & \text{, if } x > 1/a
            \end{array}\right.
        """
        return kernels_imp.tricube_pm2(z, out)


class Epanechnikov(object):
    r"""
    1D Epanechnikov density kernel with extra integrals for 1D bounded kernel estimation.
    """
    def pdf(self, xs, out=None):
        r"""
        The PDF of the kernel is usually given by:

        .. math::

            f_r(x) = \left\{\begin{array}{ll}
                            \frac{3}{4} \left(1-x^2\right) & \text{, if} x \in [-1:1]\\
                                    0 & \text{, otherwise}
                            \end{array}\right.

        As :math:`f_r` is not of variance 1 (and therefore would need adjustments for
        the bandwidth selection), we use a normalized function:

        .. math::

            f(x) = \frac{1}{\sqrt{5}}f\left(\frac{x}{\sqrt{5}}\right)
        """
        return kernels_imp.epanechnikov_pdf(xs, out)
    __call__ = pdf

    def cdf(self, xs, out=None):
        r"""
        CDF of the distribution. The CDF is defined on the interval :math:`[-\sqrt{5}:\sqrt{5}]` as:

        .. math::

            \text{cdf}(x) = \left\{\begin{array}{ll}
                    \frac{1}{2} + \frac{3}{4\sqrt{5}} x - \frac{3}{20\sqrt{5}}x^3
                    & \text{, if } x\in[-\sqrt{5}:\sqrt{5}] \\
                    0 & \text{, if } x < -\sqrt{5} \\
                    1 & \text{, if } x > \sqrt{5}
                    \end{array}\right.
        """
        return kernels_imp.epanechnikov_cdf(xs, out)

    def pm1(self, xs, out=None):
        r"""
        First partial moment of the distribution:

        .. math::

            \text{pm1}(x) = \left\{\begin{array}{ll}
                    -\frac{3\sqrt{5}}{16}\left(1-\frac{2}{5}x^2+\frac{1}{25}x^4\right)
                    & \text{, if } x\in[-\sqrt{5}:\sqrt{5}] \\
                    0 & \text{, otherwise}
                    \end{array}\right.
        """
        return kernels_imp.epanechnikov_pm1(xs, out)

    def pm2(self, xs, out=None):
        r"""
        Second partial moment of the distribution:

        .. math::

            \text{pm2}(x) = \left\{\begin{array}{ll}
                    \frac{5}{20}\left(2 + \frac{1}{\sqrt{5}}x^3 - \frac{3}{5^{5/2}}x^5 \right)
                    & \text{, if } x\in[-\sqrt{5}:\sqrt{5}] \\
                    0 & \text{, if } x < -\sqrt{5} \\
                    1 & \text{, if } x > \sqrt{5}
                    \end{array}\right.
        """
        return kernels_imp.epanechnikov_pm2(xs, out)


class Epanechnikov_order4(object):
    r"""
    Order 4 Epanechnikov kernel. That is:

    .. math::

        K_{[4]}(x) = \frac{3}{2} K(x) + \frac{1}{2} x K'(x) = -\frac{15}{8}x^2+\frac{9}{8}

    where :math:`K` is the non-normalized Epanechnikov kernel.
    """
    def pdf(self, xs, out=None):
        return kernels_imp.epanechnikov_o4_pdf(xs, out)
    __call__ = pdf

    def cdf(self, xs, out=None):
        return kernels_imp.epanechnikov_o4_cdf(xs, out)

    def pm1(self, xs, out=None):
        return kernels_imp.epanechnikov_o4_pm1(xs, out)

    def pm2(self, xs, out=None):
        return kernels_imp.epanechnikov_o4_pm2(xs, out)


class normal_order4(object):
    r"""
    Order 4 Normal kernel. That is:

    .. math::

        \phi_{[4]}(x) = \frac{3}{2} \phi(x) + \frac{1}{2} x \phi'(x) = \frac{1}{2}(3-x^2)\phi(x)

    where :math:`\phi` is the normal kernel.

    """
    def pdf(self, xs, out=None):
        return kernels_imp.normal_o4_pdf(xs, out)
    __call__ = pdf

    def cdf(self, xs, out=None):
        return kernels_imp.normal_o4_cdf(xs, out)

    def pm1(self, xs, out=None):
        return kernels_imp.normal_o4_pm1(xs, out)

    def pm2(self, xs, out=None):
        return kernels_imp.normal_o4_pm2(xs, out)

kernels1D = [normal_kernel1d, tricube, Epanechnikov, Epanechnikov_order4, normal_order4]
kernelsnD = [normal_kernel]



"""
:Author: Pierre Barbier de Reuille <pierre.barbierdereuille@gmail.com>

This module specifically implement the curve fitting, wrapping the default
scipy.optimize.leastsq function. It allows for parameter value fixing,
different kind of residual and added constraints function.
"""

from __future__ import division, print_function, absolute_import
from scipy import optimize
from numpy import array, inf
from .compat import lrange


class CurveFitting(object):
    r"""
    Fit a curve using the :py:func:`scipy.optimize.leastsq` function

    :type  xdata: ndarray
    :param xdata: Explaining values

    :type  ydata: ndarray
    :param ydata: Target values

    :type  p0: tuple
    :param p0: Initial estimates for the parameters of fct

    :type  fct: callable
    :param fct: Function to optimize. The call will be equivalent
        to ``fct(p0, xdata, *args)``

    :type  args: tuple
    :param args: Additional arguments for the function

    :type  residuals: callable or None
    :param residuals: Function computing the residuals. The call is equivalent
        to ``residuals(y, fct(x))`` and it should return a ndarray. If None,
        residuals are simply the difference between the computed and expected
        values.

    :type  fix_params: tuple of int
    :param fix_params: List of indices for the parameters in p0 that shouldn't
        change

    :type  Dfun: callable
    :param Dfun: Function computing the jacobian of fct w.r.t. the parameters.
        The call will be equivalent to ``Dfun(p0, xdata, *args)``

    :type  Dres: callable
    :param Dres: Function computing the jacobian of the residuals w.r.t. the
        parameters. The call will be equivalent to
        ``Dres(y, fct(x), DFun(x))``. If None, residuals must also be None.

    :type  col_deriv: int
    :param col_deriv: Define if Dfun returns the derivatives by row or column.
        With n = len(xdata) and m = len(p0), the shape of output of Dfun must
        be (n,m) if 0, and (m,n) if non-0.

    :type  constraints: callable
    :param constraints: If not None, this is a function that should always
        return a list ofvalues (the same), to add penalties for bad parameters.
        The function call is equivalent to: ``constraints(p0)``

    :type  lsq_args: tuple
    :param lsq_args: List of unnamed arguments passed to ``optimize.leastsq``,
        starting with ``ftol``

    :type  lsq_kword: dict
    :param lsq_kword: Dictionnary of named arguments passed to
        py:func:`scipy.optimize.leastsq`, starting with ``ftol``

    Once constructed, the following variables contain the result of
    the fitting:

    :ivar ndarray popt: The solution (or the result of the last iteration for
        an unsuccessful call)
    :ivar ndarray pcov: The estimated covariance of popt.  The diagonals
        provide the variance of the parameter estimate.
    :ivar ndarray res: Final residuals

    :ivar dict infodict: a dictionary of outputs with the keys:

            ``nfev``
                the number of function calls
            ``fvec``
                the function evaluated at the output
            ``fjac``
                A permutation of the R matrix of a QR factorization of
                the final approximate Jacobian matrix, stored column wise.
                Together with ipvt, the covariance of the estimate can be
                approximated.
            ``ipvt``
                an integer array of length N which defines a permutation
                matrix, ``p``, such that ``fjac*p = q*r``, where ``r`` is upper
                triangular with diagonal elements of nonincreasing
                magnitude. Column ``j`` of ``p`` is column ``ipvt(j)`` of the
                identity matrix.
            ``qtf``
                the vector ``(transpose(q) * fvec)``
            ``CI``
                list of tuple of parameters, each being the lower and
                upper bounds for the confidence interval in the CI
                argument at the same position.
            ``est_jacobian``
                True if the jacobian is estimated, false if the
                user-provided functions have been used

    .. note::

        In this implementation, residuals are supposed to be a generalisation
        of the notion of difference. In the end, the mathematical expression
        of this minimisation is:

        .. math::

            \hat{\theta} = \argmin_{\theta\in \mathbb{R}^p}
                \sum_i r(y_i, f(\theta, x_i))^2

        Where :math:`\theta` is the vector of :math:`p` parameters to optimise,
        :math:`r` is the residual function and :math:`f` is the function being
        fitted.
    """

    def __init__(self, xdata, ydata, p0, fct, args=(), residuals=None,
                 fix_params=(), Dfun=None, Dres = None, col_deriv=1,
                 constraints = None, *lsq_args, **lsq_kword):
        self.fct = fct
        if residuals is None:
            residuals = lambda x, y: (x - y)
            Dres = lambda y1, y0, dy: -dy

        use_derivs = (Dres is not None) and (Dfun is not None)
        df = None

        if fix_params:
            fix_params = tuple(fix_params)
            p_save = array(p0, dtype=float)
            change_params = lrange(len(p0))
            try:
                for i in fix_params:
                    change_params.remove(i)
            except ValueError:
                raise ValueError("List of parameters to fix is incorrect: "
                                 "contains either duplicates or values "
                                 "out of range.")
            p0 = p_save[change_params]

            def f(p, *args):
                p1 = array(p_save)
                p1[change_params] = p
                y0 = fct(p1, xdata, *args)
                return residuals(ydata, y0)
            if use_derivs:
                def df(p, *args):
                    p1 = array(p_save)
                    p1[change_params] = p
                    y0 = fct(p1, xdata, *args)
                    dfct = Dfun(p1, xdata, *args)
                    result = Dres(ydata, y0, dfct)
                    if col_deriv != 0:
                        return result[change_params]
                    else:
                        return result[:, change_params]
                    return result
        else:

            def f(p, *args):  # noqa
                y0 = fct(p, xdata, *args)
                return residuals(ydata, y0)
            if use_derivs:

                def df(p, *args):  # noqa
                    dfct = Dfun(p, xdata, *args)
                    y0 = fct(p, xdata, *args)
                    return Dres(ydata, y0, dfct)

        optim = optimize.leastsq(f, p0, args, full_output=1, Dfun=df,
                                 col_deriv=col_deriv, *lsq_args, **lsq_kword)
        popt, pcov, infodict, mesg, ier = optim
        #infodict['est_jacobian'] = not use_derivs

        if fix_params:
            p_save[change_params] = popt
            popt = p_save

        if not ier in [1, 2, 3, 4]:
            raise RuntimeError("Unable to determine number of fit parameters. "
                               "Error returned by scipy.optimize.leastsq:\n%s"
                               % (mesg,))

        res = residuals(ydata, fct(popt, xdata, *args))
        if (len(res) > len(p0)) and pcov is not None:
            s_sq = (res ** 2).sum() / (len(ydata) - len(p0))
            pcov = pcov * s_sq
        else:
            pcov = inf

        self.popt = popt
        self.pcov = pcov
        self.res = res
        self.infodict = infodict

    def __call__(self, xdata):
        """
        Return the value of the fitted function for each of the points in
        ``xdata``
        """
        return self.fct(self.popt, xdata)



"""
:Author: Pierre Barbier de Reuille <pierre.barbierdereuille@gmail.com>

This modules provides function for bootstrapping a regression method.
"""

from __future__ import division, print_function, absolute_import
import numpy as np
from numpy.random import randint
from scipy import optimize
from collections import namedtuple
from . import kernel_smoothing
from . import sharedmem
import multiprocessing as mp
from . import bootstrap_workers
from .compat import irange, izip


def adapt_curve_fit(fct, x, y, p0, args=(), **kwrds):
    popt, pcov = optimize.curve_fit(fct, x, y, **kwrds)
    return (popt, pcov, fct(popt, x, *args) - y)


def percentile(array, p, axis=0):
    """
    Compute the percentiles of the values in array
    """
    a = np.asarray(array).sort(axis=axis)
    n = (len(a) - 1) * p / 100
    n0 = np.floor(n)
    n1 = n0 + 1
    #print("%g percentile on %d = [%d-%d]" % (p*100, len(array), n0, n1))
    d = n - n0
    v0 = array[n0]
    v1 = array[n1]
    return v0 + d * (v1 - v0)


def bootstrap_residuals(fct, xdata, ydata, repeats=3000, residuals=None,
                        add_residual=None, correct_bias=False, **kwrds):
    """
    This implements the residual bootstrapping method for non-linear
    regression.

    :type  fct: callable
    :param fct: Function evaluating the function on xdata at least with
        ``fct(xdata)``

    :type  xdata: ndarray of shape (N,) or (k,N) for function with k predictors
    :param xdata: The independent variable where the data is measured

    :type  ydata: ndarray
    :param ydata: The dependant data

    :type  residuals: ndarray or callable or None
    :param residuals: Residuals for the estimation on each xdata. If callable,
        the call will be ``residuals(ydata, yopt)``.

    :type  repeats: int
    :param repeats: Number of repeats for the bootstrapping

    :type  add_residual: callable or None
    :param add_residual: Function that add a residual to a value. The call
        ``add_residual(ydata, residual)`` should return the new ydata, with
        the residuals 'applied'. If None, it is considered the residuals should
        simply be added.

    :type  correct_bias: boolean
    :param correct_bias: If true, the additive bias of the residuals is
        computed and restored

    :type  kwrds: dict
    :param kwrds: Dictionnary present to absorbed unknown named parameters

    :rtype: (ndarray, ndarray)
    :returns:

        1. xdata, with a new axis at position -2. This correspond to the
        'shuffled' xdata (as they are *not* shuffled here)

        2.Second item is the shuffled ydata. There is a line per repeat, each
        line is shuffled independently.

    .. todo::

        explain the method here, as well as how to create add_residual
    """
    if residuals is None:
        residuals = np.subtract

    yopt = fct(xdata)

    if not isinstance(residuals, np.ndarray):
        res = residuals(ydata, yopt)
    else:
        res = np.array(residuals)

    res -= np.mean(res)

    shuffle = randint(0, len(ydata), size=(repeats, len(ydata)))

    shuffled_res = res[shuffle]

    if correct_bias:
        kde = kernel_smoothing.LocalLinearKernel1D(xdata, res)
        bias = kde(xdata)
        shuffled_res += bias

    if add_residual is None:
        add_residual = np.add

    modified_ydata = add_residual(yopt, shuffled_res)

    return xdata[..., np.newaxis, :], modified_ydata


def bootstrap_regression(fct, xdata, ydata, repeats=3000, **kwrds):
    """
    This implements the shuffling of standard bootstrapping method for
    non-linear regression.

    :type  fct: callable
    :param fct: This is the function to optimize

    :type  xdata: ndarray of shape (N,) or (k,N) for function with k predictors
    :param xdata: The independent variable where the data is measured

    :type  ydata: ndarray
    :param ydata: The dependant data

    :type  repeats: int
    :param repeats: Number of repeats for the bootstrapping

    :type  kwrds: dict
    :param kwrds: Dictionnary to absorbed unknown named parameters

    :rtype: (ndarray, ndarray)
    :returns:
        1. The shuffled x data. The axis -2 has one element per repeat, the
        other axis are shuffled independently.

        2. The shuffled ydata. There is a line per repeat, each line is
        shuffled independently.

    .. todo::

        explain the method here
    """
    shuffle = randint(0, len(ydata), size=(repeats, len(ydata)))
    shuffled_x = xdata[..., shuffle]
    shuffled_y = ydata[shuffle]
    return shuffled_x, shuffled_y


def getCIs(CI, *arrays):
    #sorted_arrays = [ np.sort(a, axis=0) for a in arrays ]

    if not np.iterable(CI):
        CI = (CI,)

    def make_CI(a):
        return np.zeros((len(CI), 2) + a.shape[1:], dtype=float)
    CIs = tuple(make_CI(a) for a in arrays)
    for i, ci in enumerate(CI):
        ci = (100. - ci) / 2
        for cis, arr in izip(CIs, arrays):
            low = np.percentile(arr, ci, axis=0)
            high = np.percentile(arr, 100 - ci, axis=0)
            cis[i] = [low, high]

    return CIs

BootstrapResult = namedtuple('BootstrapResult', '''y_fit y_est y_eval CIs
shuffled_xs shuffled_ys full_results''')


def bootstrap(fit, xdata, ydata, CI, shuffle_method=bootstrap_residuals,
              shuffle_args=(), shuffle_kwrds={}, repeats=3000,
              eval_points=None, full_results=False, nb_workers=None,
              extra_attrs=(), fit_args=(), fit_kwrds={}):
    """
    This function implement the bootstrap algorithm for a regression algorithm.
    It is capable of spreading the load across many threads using shared memory
    and the :py:mod:`multiprocess` module.

    :type  fit: callable
    :param fit:
        Method used to compute regression. The call is::

            f = fit(xdata, ydata, *fit_args, **fit_kwrds)

        Fit should return an object that would evaluate the regression on a
        set of points. The next call will be::

            f(eval_points)

    :type  xdata: ndarray of shape (N,) or (k,N) for function with k predictors
    :param xdata: The independent variable where the data is measured

    :type  ydata: ndarray
    :param ydata: The dependant data

    :type  CI: tuple of float
    :param CI: List of percentiles to extract

    :type  shuffle_method: callable
    :param shuffle_method:
        Create shuffled dataset. The call is::

          shuffle_method(xdata, ydata, y_est, repeat=repeats, *shuffle_args,
                         **shuffle_kwrds)

        where ``y_est`` is the estimated dependant variable on the xdata.

    :type  shuffle_args: tuple
    :param shuffle_args: List of arguments for the shuffle method

    :type  shuffle_kwrds: dict
    :param shuffle_kwrds: Dictionnary of arguments for the shuffle method

    :type  repeats: int
    :param repeats: Number of repeats for the bootstraping

    :type  eval_points: ndarray or None
    :param eval_points: List of points to evaluate. If None, eval_point
        is xdata.

    :type  full_results: bool
    :param full_results: if True, output also the whole set of evaluations

    :type  nb_workers: int or None
    :param nb_worders: Number of worker threads. If None, the number of
        detected CPUs will be used. And if 1 or less, a single thread
        will be used.

    :type  extra_attrs: tuple of str
    :param extra_attrs: List of attributes of the fitting method to extract on
        top of the y values for confidence intervals

    :type  fit_args: tuple
    :param fit_args: List of extra arguments for the fit callable

    :type  fit_kwrds: dict
    :param fit_kwrds: Dictionnary of extra named arguments for the fit callable

    :rtype: :py:class:`BootstrapResult`
    :return: Estimated y on the data, on the evaluation points, the requested
        confidence intervals and, if requested, the shuffled X, Y and the full
        estimated distributions.
    """
    y_fit = fit(xdata, ydata, *fit_args, **fit_kwrds)
    shuffled_x, shuffled_y = shuffle_method(y_fit, xdata, ydata,
                                            repeats=repeats,
                                            *shuffle_args, **shuffle_kwrds)
    nx = shuffled_x.shape[-2]
    ny = shuffled_y.shape[0]
    extra_values = []
    for attr in extra_attrs:
        extra_values.append(getattr(y_fit, attr))

    if eval_points is None:
        eval_points = xdata
    if nb_workers is None:
        nb_workers = mp.cpu_count()

    multiprocess = nb_workers > 1

# Copy everything in shared mem
    if multiprocess:
        ra = sharedmem.zeros((repeats + 1, len(eval_points)), dtype=float)
        result_array = ra.np
        sx = sharedmem.array(shuffled_x)
        sy = sharedmem.array(shuffled_y)
        ep = sharedmem.array(eval_points)

        def make_ea(ev):
            return sharedmem.zeros((repeats + 1, len(ev)), dtype=float)
        eas = [make_ea(ev) for ev in extra_values]
        extra_arrays = [ea.np for ea in eas]
        pool = mp.Pool(mp.cpu_count(), bootstrap_workers.initialize_shared,
                       (nx, ny, ra, eas, sx, sy, ep, extra_attrs,
                        fit, fit_args, fit_kwrds))
    else:
        result_array = np.empty((repeats + 1, len(eval_points)), dtype=float)

        def make_ea(ev):
            return np.empty((repeats + 1, len(ev)), dtype=float)
        extra_arrays = [make_ea(ev) for ev in extra_values]
        bootstrap_workers.initialize(nx, ny, result_array, extra_arrays,
                                     shuffled_x, shuffled_y, eval_points,
                                     extra_attrs, fit, fit_args, fit_kwrds)

    result_array[0] = y_fit(eval_points)

    for ea, ev in izip(extra_arrays, extra_values):
        ea[0] = ev

    base_repeat = repeats // nb_workers
    if base_repeat * nb_workers < repeats:
        base_repeat += 1

    for i in irange(nb_workers):
        end_repeats = (i + 1) * base_repeat
        if end_repeats > repeats:
            end_repeats = repeats
        if multiprocess:
            pool.apply_async(bootstrap_workers.bootstrap_result,
                             (i, i * base_repeat, end_repeats))
        else:
            bootstrap_workers.bootstrap_result(i, i * base_repeat, end_repeats)

    if multiprocess:
        pool.close()
        pool.join()
    CIs = getCIs(CI, result_array, *extra_arrays)

    # copy the array to not return a view on a larger array
    y_eval = np.array(result_array[0])

    if not full_results:
        shuffled_y = shuffled_x = result_array = None
        extra_arrays = ()
    elif multiprocess:
        result_array = result_array.copy()  # copy in local memory
        extra_arrays = [ea.copy for ea in extra_arrays]

    return BootstrapResult(y_fit, y_fit(xdata), y_eval, CIs,
                           shuffled_x, shuffled_y, result_array)


def test():
    import quad
    from numpy.random import rand, randn
    from pylab import plot, clf, legend, arange, figure, title, show
    from curve_fitting import curve_fit

    def quadratic(x, params):
        p0, p1, p2 = params
        return p0 + p1 * x + p2 * x ** 2
    #test = quadratic
    test = quad.quadratic

    init = (10, 1, 1)
    target = np.array([10, 4, 1.2])
    print("Target parameters: {}".format(target))
    x = 6 * rand(200) - 3
    y = test(x, target) * (1 + 0.3 * randn(x.shape[0]))
    xr = arange(-3, 3, 0.01)
    yr = test(xr, target)

    print("Estimage best parameters, fixing the first one")
    popt, pcov, _, _ = curve_fit(test, x, y, init, fix_params=(0,))
    print("Best parameters: {}".format(popt))

    print("Estimate best parameters from data")
    popt, pcov, _, _ = curve_fit(test, x, y, init)
    print("Best parameters: {}".format(popt))

    figure(1)
    clf()
    plot(x, y, '+', label='data')
    plot(xr, yr, 'r', label='function')
    legend(loc='upper left')

    print("Residual bootstrap calculation")
    result_r = bootstrap(test, x, y, init, (95, 99),
                         shuffle_method=bootstrap_residuals, eval_points=xr,
                         fit=curve_fit)
    popt_r, pcov_r, res_r, CI_r, CIp_r, extra_r = result_r
    yopt_r = test(xr, popt_r)

    figure(2)
    clf()
    plot(xr, yopt_r, 'g', label='estimate')
    plot(xr, yr, 'r', label='target')
    plot(xr, CI_r[0][0], 'b--', label='95% CI')
    plot(xr, CI_r[0][1], 'b--')
    plot(xr, CI_r[1][0], 'k--', label='99% CI')
    plot(xr, CI_r[1][1], 'k--')
    legend(loc='upper left')
    title('Residual Bootstrapping')

    print("Regression bootstrap calculation")
    (popt_c, pcov_c, res_c, CI_c, CIp_r,
     extra_c) = bootstrap(test, x, y, init, CI=(95, 99),
                          shuffle_method=bootstrap_regression, eval_points=xr,
                          fit=curve_fit)
    yopt_c = test(xr, popt_c)

    figure(3)
    clf()
    plot(xr, yopt_c, 'g', label='estimate')
    plot(xr, yr, 'r', label='target')
    plot(xr, CI_c[0][0], 'b--', label='95% CI')
    plot(xr, CI_c[0][1], 'b--')
    plot(xr, CI_c[1][0], 'k--', label='99% CI')
    plot(xr, CI_c[1][1], 'k--')
    legend(loc='upper left')
    title('Regression Bootstrapping (also called Case Resampling)')

    print("Done")

    show()

    return locals()


def profile(filename='bootstrap_profile'):
    import cProfile
    import pstats
    cProfile.run('res = bootstrap.test()', 'bootstrap_profile')
    p = pstats.Stats('bootstrap_profile')
    return p

if __name__ == "__main__":
    test()



from __future__ import division, absolute_import, print_function

from .. import kernels
from .. import _kernels

from scipy import stats
from scipy.fftpack import fft, dct
from scipy.integrate import quad
import numpy as np


class KernelTester(object):
    @classmethod
    def initUpClass(cls, lower=-np.inf, test_width=3):
        cls.lower = float(lower)
        cls.hard_points = ()
        cls.quad_args = dict(limit=100)
        cls.test_xs = np.r_[-test_width / 2:test_width / 2:17j]
        cls.pos_xs = 20 * (np.arange(1024) + 0.5) / 1024
        cls.xs = np.roll(20 * (np.arange(1024) - 512) / 1024, 512)
        cls.pos_freqs = np.pi / 20 * np.arange(1024)
        freqs = 2 * np.pi / 20 * (np.arange(1024) - 512)
        freqs = np.roll(freqs, 512)
        cls.freqs = freqs

    def _cdf(self, upper, tol):
        val = self.kernel.cdf(upper)
        args = dict(self.quad_args)
        if self.hard_points:
            args.update(points=[p for p in self.hard_points if p > self.lower and p < upper])
        val1, err = quad(self.kernel.pdf, self.lower, upper, **args)
        assert abs(val - val1) < tol, "cdf({3:.9g}) -- Expected: {0:g}, computed: {1:g}".format(val1, val, err, upper)

    def _pm1(self, upper, tol):
        val = self.kernel.pm1(upper)

        def fct(x):
            return x * self.kernel.pdf(x)
        args = dict(self.quad_args)
        if self.hard_points:
            args.update(points=[p for p in self.hard_points if p > self.lower and p < upper])
        val1, err = quad(fct, self.lower, upper, **args)
        if err < 1e-8:
            err = 1e-8
        assert abs(val - val1) < tol, "pm1({3:.9g}) -- Expected: {0:g}, computed: {1:g} (tol= {4:g})".format(val1, val, err, upper, tol)

    def _pm2(self, upper, tol):
        val = self.kernel.pm2(upper)

        def fct(x):
            return x * x * self.kernel.pdf(x)
        args = dict(self.quad_args)
        if self.hard_points:
            args.update(points=[p for p in self.hard_points if p > self.lower and p < upper])
        val1, err = quad(fct, self.lower, upper, **args)
        if err < 1e-8:
            err = 1e-8
        assert abs(val - val1) < tol, "pm2({3:.9g}) -- Expected: {0:g}, computed: {1:g}".format(val1, val, err, upper)

    def test_cdf_fct(self, tol=1e-5):
        for x in self.test_xs:
            yield self._cdf, x, tol

    def test_pm1_fct(self, tol=1e-5):
        for x in self.test_xs:
            yield self._pm1, x, tol

    def test_pm2_fct(self, tol=1e-5):
        for x in self.test_xs:
            yield self._pm2, x, tol

    def test_dct_fct(self):
        if hasattr(self.kernel, 'dct'):
            k = self.kernel
            p = k.pdf(self.pos_xs)
            cdt_ref = dct(p * (self.pos_xs[1] - self.pos_xs[0]))
            cdt_tst = k.dct(self.pos_freqs)
            np.testing.assert_array_almost_equal(cdt_ref, cdt_tst, 8)

    def test_fft_fct(self):
        if hasattr(self.kernel, 'fft'):
            k = self.kernel
            p = k.pdf(self.xs)
            fft_ref = fft(p * (self.xs[1] - self.xs[0])).real
            fft_tst = k.fft(self.freqs)
            np.testing.assert_array_almost_equal(fft_ref, fft_tst, 10)


class TestNormal1d(KernelTester):

    @classmethod
    def setUpClass(cls):
        cls.kernel = kernels.normal_kernel1d()
        cls.initUpClass()
        cls.norm_ref = stats.norm(loc=0, scale=1)

    def attr(self, attr):
        n_ref = self.norm_ref
        n_tst = self.kernel
        ref_vals = getattr(n_ref, attr)(self.xs)
        tst_vals = getattr(n_tst, attr)(self.xs)
        np.testing.assert_array_almost_equal(ref_vals, tst_vals, 10)

    def test_pdf(self):
        self.attr('pdf')

    def test_cdf(self):
        self.attr('cdf')

    def test_dct(self):
        n_ref = self.norm_ref
        n_tst = self.kernel
        p = n_ref.pdf(self.pos_xs)
        cdt_ref = dct(p * (self.pos_xs[1] - self.pos_xs[0]))
        cdt_tst = n_tst.dct(self.pos_freqs)
        np.testing.assert_array_almost_equal(cdt_ref, cdt_tst, 8)

    def test_fft(self):
        n_ref = self.norm_ref
        n_tst = self.kernel
        p = n_ref.pdf(self.xs)
        fft_ref = fft(p * (self.xs[1] - self.xs[0])).real
        fft_tst = n_tst.fft(self.freqs)
        np.testing.assert_array_almost_equal(fft_ref, fft_tst, 10)


class TestEpanechnikov(KernelTester):
    @classmethod
    def setUpClass(cls):
        cls.initUpClass()
        cls.kernel = kernels.Epanechnikov()


class TestTricube(KernelTester):
    @classmethod
    def setUpClass(cls):
        cls.initUpClass(lower=-5)
        cls.kernel = kernels.tricube()
        cls.hard_points = (-1 / _kernels.tricube_width, 1 / _kernels.tricube_width)


class TestEpanechnikov_order4(KernelTester):
    @classmethod
    def setUpClass(cls):
        cls.initUpClass(lower=-5)
        cls.kernel = kernels.Epanechnikov_order4()


class Testnormal_order4(KernelTester):
    @classmethod
    def setUpClass(cls):
        cls.initUpClass()
        cls.kernel = kernels.normal_order4()


class ComparePythonCython(object):
    @classmethod
    def initUpClass(cls):
        cls.xs = np.r_[-3:3:4096j]

    def _pdf(self, k):
        kernels.useCython()
        cy = k.pdf(self.xs)
        kernels.usePython()
        py = k.pdf(self.xs)
        np.testing.assert_array_almost_equal(cy, py, 7)

    def testing_pdf(self):
        for cls in kernels.kernels1D:
            yield self._pdf, cls

    def _cdf(self, k):
        kernels.useCython()
        cy = k.cdf(self.xs)
        kernels.usePython()
        py = k.cdf(self.xs)
        np.testing.assert_array_almost_equal(cy, py, 7)

    def testing_cdf(self):
        for cls in kernels.kernels1D:
            yield self._cdf, cls

    def _pm1(self, k):
        kernels.useCython()
        cy = k.pm1(self.xs)
        kernels.usePython()
        py = k.pm1(self.xs)
        np.testing.assert_array_almost_equal(cy, py, 7)

    def testing_pm1(self):
        for cls in kernels.kernels1D:
            yield self._pm1, cls

    def _pm2(self, k):
        kernels.useCython()
        cy = k.pm2(self.xs)
        kernels.usePython()
        py = k.pm2(self.xs)
        np.testing.assert_array_almost_equal(cy, py, 7)

    def testing_pm2(self):
        for cls in kernels.kernels1D:
            yield self._pm2, cls



from __future__ import division
from numpy import exp, argsort, ones, log


class Linear(object):
    name = "Linear"
    description = "y = a x + b"
    args = "a b".split()

    @staticmethod
    def __call__(ab, x):
        a, b = ab
        return a * x + b

    @staticmethod
    def Dfun(ab, x):
        a, b = ab
        result = ones((2, x.shape[0]), dtype=x.dtype)
        result[0] = x  # d/da
        # d/db = 1
        return result

    @staticmethod
    def init_args(x, y):
        b = y.min()
        a = (y.max() - y.min()) / (x.max() - x.min())
        return a, b


class Exponential(object):
    description = "y = A e^{k(x-x_0)} + y_0"
    args = "A k x_0 y_0".split()
    name = "Exponential"

    @staticmethod
    def __call__(parms, x):
        (A, k, x0, y0) = parms
        return A * exp(k * (x - x0)) + y0

    @staticmethod
    def Dfun(parms, x):
        (A, k, x0, y0) = parms
        result = ones((4, x.shape[0]), dtype=x.dtype)
        dx = x - x0
        ee = exp(k * dx)
        result[0] = ee  # d/dA
        result[1] = dx * A * ee  # d/dk
        result[2] = -A * ee  # d/dx0
        # d/dy0 = 1
        return result

    @staticmethod
    def init_args(x, y):
        x0 = (x.max() + x.min()) / 2
        IX = argsort(x)
        xs = x[IX]
        ys = y[IX]
        k = log(ys[-1] / ys[0]) / (xs[-1] - xs[0])
        A = ys[-1] / (exp(k * (xs[-1] - x0)))
        return A, k, x0, ys[0]


class PowerLaw(object):
    description = "y = A (x-x_0)^k + y_0"
    name = "Power law"
    args = "A x_0 k y_0".split()

    @staticmethod
    def __call__(parms, x):
        (A, x0, k, y0) = parms
        return A * (x - x0) ** k + y0

    @staticmethod
    def Dfun(parms, x):
        (A, x0, k, y0) = parms
        result = ones((4, x.shape[0]), dtype=x.dtype)
        dx = x - x0
        dxk1 = dx ** (k - 1)
        dxk = dxk1 * dx
        result[0] = dxk  # d/dA
        result[1] = -A * k * dxk1  # d/dx0
        result[2] = dxk * A * log(dx)  # d/dk
        # d/dy0 = 1
        return result

    @staticmethod
    def init_args(x, y):
        x0 = x.min()
        y0 = y.min()
        A = (y.max() - y0) / (x.max() - x0)
        return (A, x0, 1, y0)


class Logistic(object):
    description = "y = A / (1 + e^{-k (x-x_0)}) + y_0"
    args = "A k x_0 y_0".split()
    name = "Logistic"

    @staticmethod
    def __call__(parms, x):
        (A, k, x0, y0) = parms
        return A / (1 + exp(k * (x0 - x))) + y0

    @staticmethod
    def Dfun(parms, x):
        (A, k, x0, y0) = parms
        result = ones((4, x.shape[0]), dtype=x.dtype)
        dx = x0 - x
        ee = exp(k * dx)
        ee1 = ee + 1.
        ee2 = ee1 * ee1
        result[0] = 1. / ee1  # d/dA
        result[1] = -dx * A * ee / ee2  # d/dk
        result[2] = -A * k * ee / ee2  # d/dx0
        # d/dy0 = 1
        return result

    @staticmethod
    def init_args(x, y):
        x0 = (x.max() + x.min()) / 2
        k = log(y.min()) / (x.min() - x0)
        return y.max(), k, x0, y.min()



from __future__ import division, print_function, absolute_import
import sys
import csv

PY2 = sys.version_info[0] == 2

if PY2:
    user_text = unicode
    text_type = basestring
    unichr = unichr
    irange = xrange
    lrange = range
    CSV_READ_FLAGS = b"rb"
    DECODE_STRING = lambda s: s.decode('utf_8')
    from itertools import izip

    def unicode_csv_reader(unicode_csv_data, dialect=csv.excel, **kwargs):
        # csv.py doesn't do Unicode; encode temporarily as UTF-8:
        csv_reader = csv.reader(unicode_csv_data,
                                dialect=dialect, **kwargs)
        for row in csv_reader:
            # decode UTF-8 back to Unicode, cell by cell:
            yield [unicode(cell, 'utf-8') for cell in row]

    class unicode_csv_writer(object):
        def __init__(self, *args, **kwords):
            self.csv = csv.writer(*args, **kwords)

        def writerows(self, rows):
            rows = [[unicode(val).encode('utf-8') for val in row]
                    for row in rows]
            return self.csv.writerows(rows)

        def writerow(self, row):
            row = [unicode(val).encode('utf-8') for val in row]
            return self.csv.writerow(row)
else:
    user_text = str
    text_type = str
    unichr = chr
    irange = range
    lrange = lambda x: list(range(x))
    CSV_READ_FLAGS = u"rt"
    DECODE_STRING = lambda s: s
    izip = zip
    unicode_csv_reader = csv.reader
    unicode_csv_writer = csv.writer



#cython: boundscheck=False
#cython: wraparound=False

import numpy as np
cimport numpy as np
from math cimport isfinite, powl, expl, sqrtl, lgammal, logl
from libc.math cimport erf, exp, sqrt, M_PI

ctypedef np.npy_float64 float64_t
ctypedef np.npy_float128 float128_t
ctypedef np.npy_int int_t
ctypedef np.npy_int64 int64_t

cdef float128_t M_PIl = 3.1415926535897932384626433832795029

cdef float64_t S2PI = sqrt(2.0*M_PI)
cdef float128_t S2PIl = sqrtl(2.0*M_PIl)
cdef float64_t S2 = sqrt(2.0)
cdef float128_t S2l = sqrtl(2.0)

s2pi = S2PI
s2 = S2

def f_comp_(object I, object a2, float128_t s, float128_t t):
    return np.float128(f_comp(I, a2, s, t))

def lgamma(long double v):
    return lgammal(v)

cdef long double _odd_factorial(int k):
    """
    Odd factorial of (2*k-1) => (2k-1)!! = (2k)!/(2^k k!)
    """
    return expl(lgammal(2.*k+1.)-((logl(2.) * k) + lgammal(k+1.)))

def odd_factorial(int k):
    return _odd_factorial(k)

def ref_odd_factorial(int k):
    cdef int result = 1
    for i from 2 <= i <= k:
        result *= 2*i-1
    return result

cdef float128_t _fixed_point(float64_t t, float128_t M, np.ndarray[float128_t] I, np.ndarray[float128_t] a2):
    cdef int l=7
    cdef int s
    cdef long double K0 = _odd_factorial(l)/S2PIl
    cdef long double cst, time
    cdef float128_t f = f_comp(I, a2, l, t)
    for s from l >= s > 1:
        cst = (1. + powl(0.5, s + .5))/3.0
        time=powl(2*cst*K0/M/f,2./(3.+2.*s))
        f=f_comp(I, a2, s, time)
        K0 /= 2*s-1
    return t-powl(2*M*sqrtl(M_PI)*f,-2./5)

def fixed_point(object t, object M_, object I_, object a2_):
    cdef np.ndarray[float128_t] I = np.float128(I_)
    cdef float128_t M = M_
    cdef np.ndarray[float128_t] a2 = np.float128(a2_)
    return np.float128(_fixed_point(t, M, I, a2))





#cython: boundscheck=False
#cython: wraparound=False

import numpy as np
cimport numpy as np
from math cimport isfinite, erf, fabs
from libc.math cimport exp, sqrt, M_PI, pow

np.import_array()

ctypedef np.npy_float64 float64_t
#ctypedef np.npy_float128 float128_t

cdef float64_t S2PI = sqrt(2.0*M_PI)
cdef float64_t S2 = sqrt(2.0)

s2pi = S2PI
s2 = S2

cdef void _vectorize(object z,
                     object out,
                     float64_t (*fct)(float64_t v)):
    cdef np.broadcast it = np.broadcast(z, out)
    while np.PyArray_MultiIter_NOTDONE(it):
        (<float64_t*> np.PyArray_MultiIter_DATA(it, 1))[0] = fct((<float64_t*> np.PyArray_MultiIter_DATA(it, 0))[0])
        np.PyArray_MultiIter_NEXT(it)

cdef object vectorize(object z,
                      object out,
                      float64_t (*fct)(float64_t v)):
    cdef np.ndarray zz = np.asfarray(z)
    if zz.ndim > 0:
        if out is None:
            out = np.PyArray_EMPTY(zz.ndim, zz.shape, np.NPY_FLOAT64, False)
        _vectorize(<object>zz, out, fct)
        return out
    else:
        return fct(<float64_t>zz)

cdef float64_t _norm1d_pdf(float64_t z):
    return exp(-z*z/2)/S2PI

def norm1d_pdf(object z, object out = None):
    return vectorize(z, out, _norm1d_pdf)

cdef float64_t _norm1d_cdf(float64_t z):
    return erf(z/S2) / 2 + 0.5

def norm1d_cdf(object z, object out = None):
    return vectorize(z, out, _norm1d_cdf)

cdef float64_t _norm1d_pm1(float64_t z):
    return -exp(-z*z/2) / S2PI

def norm1d_pm1(object z, object out = None):
    return vectorize(z, out, _norm1d_pm1)

cdef float64_t _norm1d_pm2(float64_t z):
    if isfinite(z):
        return 0.5*erf(z/S2) + 0.5 - z/S2PI*exp(-z*z/2)
    return 0.5*erf(z/S2)+0.5

def norm1d_pm2(object z, object out = None):
    return vectorize(z, out, _norm1d_pm2)

cdef float64_t tricube_a = sqrt(35./243)
#cdef float128_t tricube_al = sqrtl(35./243)
tricube_width = tricube_a

cdef float64_t _tricube_pdf(float64_t z):
    z *= tricube_a
    if z < -1 or z > 1:
        return 0
    return 70./81*pow(1 - pow(fabs(z), 3.), 3.) * tricube_a

def tricube_pdf(object z, object out = None):
    return vectorize(z, out, _tricube_pdf)

cdef float64_t _tricube_cdf(float64_t z):
    z *= tricube_a
    if z < -1:
        return 0.
    if z > 1:
        return 1.
    if z > 0:
        return 1./162*(60*pow(z, 7.) - 7.*(2*pow(z, 10.) + 15.*pow(z, 4.)) + 140*z + 81)
    else:
        return 1./162*(60*pow(z, 7.) + 7.*(2*pow(z, 10.) + 15.*pow(z, 4.)) + 140*z + 81)

def tricube_cdf(object z, object out = None):
    return vectorize(z, out, _tricube_cdf)

cdef float64_t _tricube_pm1(float64_t zc):
    cdef float64_t z = zc
    z *= tricube_a
    if z < -1 or z > 1:
        return 0
    if z < 0:
        z = -z
    cdef float64_t z2 = z*z
    cdef float64_t z3 = z2*z
    cdef float64_t z5 = z3*z2
    cdef float64_t z8 = z5*z3
    cdef float64_t z11 = z8*z3
    return 7./(tricube_a*3564)*(165*z8 - 8.*(5*z11 + 33.*z5) + 220*z2 - 81)

def tricube_pm1(object z, object out = None):
    return vectorize(z, out, _tricube_pm1)

cdef float64_t _tricube_pm2(float64_t z):
    z *= tricube_a
    if z < -1:
        return 0
    if z > 1:
        return 1
    cdef float64_t z3 = z*z*z
    cdef float64_t z6 = z3*z3
    cdef float64_t z9 = z6*z3
    cdef float64_t z12 = z9*z3
    if z > 0:
        return 35./(tricube_a*tricube_a*486)*(4*z9 - (z12 + 6.*z6) + 4*z3 + 1)
    else:
        return 35./(tricube_a*tricube_a*486)*(4*z9 + (z12 + 6.*z6) + 4*z3 + 1)

def tricube_pm2(object z, object out = None):
    return vectorize(z, out, _tricube_pm2)

cdef float64_t epanechnikov_a = 1./sqrt(5.)
epanechnikov_width = epanechnikov_a

cdef float64_t _epanechnikov_pdf(float64_t z):
    z *= epanechnikov_a
    if z < -1:
        return 0
    if z > 1:
        return 0
    return .75*(1-z*z)*epanechnikov_a

def epanechnikov_pdf(object z, object out = None):
    return vectorize(z, out, _epanechnikov_pdf)

cdef float64_t _epanechnikov_cdf(float64_t z):
    z *= epanechnikov_a
    if z < -1:
        return 0
    if z > 1:
        return 1
    return .25*(2+3*z-z*z*z)

def epanechnikov_cdf(object z, object out = None):
    return vectorize(z, out, _epanechnikov_cdf)

cdef float64_t _epanechnikov_pm1(float64_t z):
    z *= epanechnikov_a
    if z < -1:
        return 0
    if z > 1:
        return 0
    cdef float64_t z2 = z*z
    return -3./16.*(1-2*z2+z2*z2)/epanechnikov_a

def epanechnikov_pm1(object z, object out = None):
    return vectorize(z, out, _epanechnikov_pm1)

cdef float64_t _epanechnikov_pm2(float64_t z):
    z *= epanechnikov_a
    if z < -1:
        return 0
    if z > 1:
        return 1
    cdef float64_t z3 = z*z*z
    return 0.25*(2+5*z3-3*z3*z*z)

def epanechnikov_pm2(object z, object out = None):
    return vectorize(z, out, _epanechnikov_pm2)

cdef float64_t _epanechnikov_o4_pdf(float64_t z):
    if z < -1:
        return 0
    if z > 1:
        return 0
    return 0.125*(9-15*z*z)

def epanechnikov_o4_pdf(object z, object out = None):
    return vectorize(z, out, _epanechnikov_o4_pdf)

cdef float64_t _epanechnikov_o4_cdf(float64_t z):
    if z < -1:
        return 0
    if z > 1:
        return 1
    return .125*(4+9*z-5*z*z*z)

def epanechnikov_o4_cdf(object z, object out = None):
    return vectorize(z, out, _epanechnikov_o4_cdf)

cdef float64_t _epanechnikov_o4_pm1(float64_t z):
    if z < -1:
        return 0
    if z > 1:
        return 0
    cdef float64_t z2 = z*z
    return 1./32.*(18*z2-3-15*z2*z2)

def epanechnikov_o4_pm1(object z, object out = None):
    return vectorize(z, out, _epanechnikov_o4_pm1)

cdef float64_t _epanechnikov_o4_pm2(float64_t z):
    if z < -1:
        return 0
    if z > 1:
        return 0
    cdef float64_t z2 = z*z
    cdef float64_t z3 = z2*z
    return .375*(z3 - z2*z3)

def epanechnikov_o4_pm2(object z, object out = None):
    return vectorize(z, out, _epanechnikov_o4_pm2)

cdef float64_t _normal_o4_pdf(float64_t z):
    return (3-z*z)*_norm1d_pdf(z)/2

def normal_o4_pdf(object z, object out = None):
    return vectorize(z, out, _normal_o4_pdf)

cdef float64_t _normal_o4_cdf(float64_t z):
    return _norm1d_cdf(z)+z*_norm1d_pdf(z)/2

def normal_o4_cdf(object z, object out = None):
    return vectorize(z, out, _normal_o4_cdf)

cdef float64_t _normal_o4_pm1(float64_t z):
    return _norm1d_pdf(z) - _normal_o4_pdf(z)

def normal_o4_pm1(object z, object out = None):
    return vectorize(z, out, _normal_o4_pm1)

cdef float64_t _normal_o4_pm2(float64_t z):
    return z*z*z/2*_norm1d_pdf(z)

def normal_o4_pm2(object z, object out = None):
    return vectorize(z, out, _normal_o4_pm2)




from __future__ import division, absolute_import, print_function

__all__ = ["test_bootstrap",
           "test_bootstrap_workers",
           "test_curve_fitting",
           "test_cyth",
           "test_kde",
           "test_kernel_smoothing",
           "test_kernels",
           "test_loader",
           "test_plot_fit",
           "test_pyqt_fit1d",
           "test_sharedmem",
           "test_utils"]



#!/usr/bin/env python
from __future__ import division, print_function, absolute_import
from . import functions, residuals, plot_fit, bootstrap
from .compat import user_text, CSV_READ_FLAGS
from .compat import unicode_csv_reader as csv_reader

from PyQt4 import QtGui, QtCore, uic
from PyQt4.QtCore import pyqtSignature, Qt
from PyQt4.QtGui import QMessageBox
import matplotlib
from numpy import nan, array, ma, arange
from path import path
from .curve_fitting import CurveFitting
import sys
from pylab import close as close_figure
import traceback
import re

CIsplitting = re.compile(r'[;, :-]')


def get_args(*a, **k):
    return a, k


def find(array):
    return arange(len(array))[array]


class ParametersModel(QtCore.QAbstractTableModel):
    def __init__(self, data, function, res, idxX, idxY, parent=None):
        QtCore.QAbstractTableModel.__init__(self, parent)
        values = data[:, [idxX, idxY]]
        values = values.data[values.mask.sum(axis=1) == 0]
        self.valuesX = values[:, 0]
        self.valuesY = values[:, 1]
        self.fct = function
        self.res = res
        self.parm_names = function.args
        self.parm_values = list(function.init_args(self.valuesX, self.valuesY))
        self.fixed = [False] * len(function.args)

    def rowCount(self, idx=QtCore.QModelIndex()):
        return len(self.parm_names)

    def columnCount(self, idx=QtCore.QModelIndex()):
        return 3

    def headerData(self, section, orientation, role=Qt.DisplayRole):
        if role != Qt.DisplayRole:
            return
        if orientation == Qt.Horizontal:
            if section == 0:
                return "Parameter"
            elif section == 1:
                return "Value"
            elif section == 2:
                return "Fixed"

    def flags(self, index):
        if index.column() == 0:
            return Qt.ItemIsEnabled
        elif index.column() == 1:
            return Qt.ItemIsEnabled | Qt.ItemIsEditable
        elif index.column() == 2:
            return Qt.ItemIsEnabled | Qt.ItemIsUserCheckable
        return Qt.NoItemFlags

    def data(self, index, role=Qt.DisplayRole):
        r = index.row()
        c = index.column()
        if 0 <= r < len(self.parm_names) and 0 <= c < 3:
            if c == 0:
                if role == Qt.DisplayRole:
                    return self.parm_names[r]
            elif c == 1:
                if role == Qt.DisplayRole:
                    return "%g" % (self.parm_values[r],)
                elif role == Qt.EditRole:
                    return "%g" % (self.parm_values[r],)
            elif c == 2:
                if role == Qt.CheckStateRole:
                    return self.fixed[r]

    def setData(self, index, value, role=Qt.DisplayRole):
        r = index.row()
        c = index.column()
        if 0 <= r < len(self.parm_names) and 0 < c < 3:
            if c == 1 and role == Qt.EditRole:
                f, ok = value.toDouble()
                if ok:
                    self.parm_values[r] = f
                    self.dataChanged.emit(index, index)
                    return True
                else:
                    print("Error, cannot convert value to double")
            elif c == 2 and role == Qt.CheckStateRole:
                self.fixed[r] = value
                self.dataChanged.emit(index, index)
                return True
        return False


class QtFitDlg(QtGui.QDialog):
    def __init__(self, *args, **kwords):
        QtGui.QDialog.__init__(self, *args, **kwords)
        p = (path(__file__).dirname() / 'qt_fit.ui').abspath()
        uic.loadUi(p, self)
        if sys.platform != "darwin":
            self.selectInputFile.setMaximumWidth(32)
            self.selectOutputFile.setMaximumWidth(32)
        self.validator = QtGui.QDoubleValidator()
        self.xMin.setValidator(self.validator)
        self.xMax.setValidator(self.validator)
        self.buttonBox.addButton("Plot", QtGui.QDialogButtonBox.ApplyRole)
        self.buttonBox.addButton("Close Plots", QtGui.QDialogButtonBox.ResetRole)
        self.init()

    def init(self):
        self._fct = None
        self._parameters = None
        self._res = None
        self._data = None
        self._fieldX = None
        self._fieldY = None
        self._input = None
        self._output = None
        self._CI = None
        self._scale = None
        self._header = None
        self._CIchanged = False
        self._write = False
        self.setData(None, None)
        residuals.load()
        functions.load()
        list_fcts = sorted(functions.names())
        list_res = sorted(residuals.names())
        self.function.clear()
        self.function.addItems(list_fcts)
        self.residuals.clear()
        self.residuals.addItems(list_res)
        self.residuals.setCurrentIndex(list_res.index("Standard"))
        self.on_computeCI_toggled(self.computeCI.isChecked())

    @pyqtSignature("const QString&")
    def on_function_currentIndexChanged(self, txt):
        print("New function: {}".format(txt))
        self.fct = functions.get(str(txt))

    @pyqtSignature("const QString&")
    def on_residuals_currentIndexChanged(self, txt):
        print("New residual: {}".format(txt))
        self.res = residuals.get(str(txt))

    @pyqtSignature("")
    def on_selectInputFile_clicked(self):
        filename = QtGui.QFileDialog.getOpenFileName(self, "Open CSV file",
                                                     filter="CSV file (*.csv);;All Files (*.*)")
        if filename:
            self.input = filename

    @pyqtSignature("")
    def on_selectOutputFile_clicked(self):
        filename = QtGui.QFileDialog.getSaveFileName(self, "Save CSV file",
                                                     filter="CSV file (*.csv);;All Files (*.*)")
        if filename:
            self.output = filename

    @pyqtSignature("const QString&")
    def on_fieldXbox_currentIndexChanged(self, txt):
        self.fieldX = txt

    @pyqtSignature("const QString&")
    def on_fieldYbox_currentIndexChanged(self, txt):
        self.fieldY = txt

    def _getFct(self):
        return self._fct

    def _setFct(self, f):
        if f != self._fct:
            self._fct = f
            if self.function.currentText() != f.name:
                self.function.setCurrentIndex(self.function.findText(f.name))
            if self.input:
                self.updateParameters()

    fct = property(_getFct, _setFct)

    def _getRes(self):
        return self._res

    def _setRes(self, res):
        if res != self._res:
            self._res = res
            if self.residuals.currentText() != res.name:
                self.residuals.setCurrentIndex(self.residuals.findText(res.name))
    res = property(_getRes, _setRes)

    @pyqtSignature("const QString&")
    def on_inputFile_textChanged(self, txt):
        txt = path(txt)
        self.input = txt

    def _getInput(self):
        return self._input

    def _setInput(self, txt):
        txt = path(txt)
        if txt != self._input and txt.isfile():
            try:
                data = None
                header = None
                with open(txt, CSV_READ_FLAGS) as f:
                    try:
                        r = csv_reader(f)
                        header = next(r)
                        if len(header) < 2:
                            QMessageBox.critical(self, "Error reading CSV file",
                                                 "Error, the file doesn't have at least 2 columns")
                            return
                        data = []
                        for line in r:
                            if not line:
                                break
                            data.append([float(field) if field else nan for field in line])
                        max_length = max(len(l) for l in data)
                        data = array([line + [nan] * (max_length - len(line)) for line in data],
                                     dtype=float)
                        data = ma.masked_invalid(data)
                    except Exception as ex:
                        QMessageBox.critical(self, "Error reading CSV file", str(ex))
                        data = None
                        header = None
                if data is not None:
                    self._input = txt
                    print("input: {}".format(self._input))
                    if self._input != self.inputFile.text():
                        self.inputFile.setText(self._input)
                    self.setData(header, data)
            except IOError:
                pass
    input = property(_getInput, _setInput)

    def setData(self, header, data):
        if header is None or data is None:
            self._header = None
            self._data = None
            self.parameters.setModel(None)
            self.param_model = None
        else:
            self._header = header
            self._data = data
            self.fieldXbox.clear()
            self.fieldXbox.addItems(self._header)
            self.fieldYbox.clear()
            self.fieldYbox.addItems(self._header)
            self.fieldX = self._header[0]
            self.fieldY = self._header[1]

    def _getOutput(self):
        return self._output

    def _setOutput(self, txt):
        txt = path(txt)
        if self._output != txt:
            if txt and not txt.endswith(".csv"):
                txt += ".csv"
            self._output = txt
            if self._output != self.outputFile.text():
                self.outputFile.setText(self._output)
    output = property(_getOutput, _setOutput)

    @pyqtSignature("const QString&")
    def on_outputFile_textChanged(self, txt):
        self.output = txt

    def _getWriteResult(self):
        return self._write

    def _setWriteResult(self, on):
        on = bool(on)
        if on != self._write:
            self._write = on
            self.writeOutput.setChecked(on)
    writeResult = property(_getWriteResult, _setWriteResult)

    @pyqtSignature("bool")
    def on_writeOutput_toggled(self, on):
        self.writeResult = on

    def _getHeader(self):
        return self._header
    header = property(_getHeader)

    def _getFieldX(self):
        return self._fieldX

    def _setFieldX(self, txt):
        if txt != self._fieldX and txt in self.header:
            self._fieldX = txt
            if txt != self.fieldXbox.currentText():
                self.fieldXbox.setCurrentIndex(self.fieldXbox.findText(txt))
            self.updateParameters()
    fieldX = property(_getFieldX, _setFieldX)

    def _getFieldY(self):
        return self._fieldY

    def _setFieldY(self, txt):
        if txt != self._fieldY and txt in self.header:
            self._fieldY = txt
            if txt != self.fieldYbox.currentText():
                self.fieldYbox.setCurrentIndex(self.fieldYbox.findText(txt))
            self.updateParameters()
    fieldY = property(_getFieldY, _setFieldY)

    def updateParameters(self):
        if self._data is not None and \
                self.fct is not None and \
                self.res is not None and \
                self.fieldX is not None \
                and self.fieldY is not None:
            idxX = self.header.index(user_text(self.fieldX))
            idxY = self.header.index(user_text(self.fieldY))
            self.param_model = ParametersModel(self._data, self.fct, self.res, idxX, idxY)
            self.parameters.setModel(self.param_model)
            minx = self._data[:, idxX].min()
            maxx = self._data[:, idxX].max()
            self.xMin.setText(str(minx))
            self.xMax.setText(str(maxx))
        #elif self._data is None:
            #print "Missing data"
        #elif self.function is None:
            #print "Missing function"
        #elif self.res is None:
            #print "Missing res"
        #elif self.fieldX is None:
            #print "Missing fieldX"
        #elif self.fieldY is None:
            #print "Missing fieldY"

    @pyqtSignature("bool")
    def on_computeCI_toggled(self, on):
        if on:
            meth = self.CImethod.currentText()
            ints = [float(f) for f in CIsplitting.split(user_text(self.CIvalues.text())) if f]
            self.CI = [meth, ints]
        else:
            self.CI = None

    @pyqtSignature("const QString&")
    def on_CIvalues_textEdited(self, txt):
        self._CIchanged = True

    @pyqtSignature("")
    def on_CIvalues_editingFinished(self):
        if self.CI:
            try:
                ints = [float(f) for f in CIsplitting.split(user_text(self.CIvalues.text())) if f]
                self.setIntervals(ints)
            except:
                pass
            if self.CI[1]:
                self.CIvalues.setText(";".join("{:g}".format(f) for f in self.CI[1]))
            else:
                self.CIvalues.setText("")
            self._CIchanged = False

    @pyqtSignature("const QString&")
    def on_CImethod_currentIndexChanged(self, txt):
        if self.CI:
            meth = user_text(txt)
            self.setCIMethod(meth)

    def _getCI(self):
        return self._CI

    def _setCI(self, val):
        if val is not None:
            val = (user_text(val[0]), [float(f) for f in val[1]])
        if val != self._CI:
            self._CI = val
            if val is not None:
                meth, ints = val
                if meth != self.CImethod.currentText():
                    self.CImethod.setCurrentIndex(self.CImethod.findText(meth))
                self.CIvalues.setText(";".join("{:g}".format(f) for f in ints))
    CI = property(_getCI, _setCI)

    def setCIMethod(self, meth):
        if meth != self._CI[0]:
            self._CI = (meth, self._CI[1])
            if meth != self.CImethod.currentText():
                self.CImethod.setCurrentIndex(self.CImethod.findText(meth))

    def setIntervals(self, ints):
        if ints != self._CI[1]:
            self._CI = (self._CI[0], ints)
            self.CIvalues.setText(";".join("{:g}".format(f) for f in ints))

    @pyqtSignature("QAbstractButton*")
    def on_buttonBox_clicked(self, button):
        role = self.buttonBox.buttonRole(button)
        if role == QtGui.QDialogButtonBox.ResetRole:
            close_figure('all')
        elif role == QtGui.QDialogButtonBox.ApplyRole:
            self.plot()

    @pyqtSignature("")
    def on_buttonBox_rejected(self):
        close_figure('all')

    def plot(self):
        if self.param_model is None:
            QMessageBox.critical(self, "Error plotting", "Error, you don't have any data loaded")
        else:
            if self._CIchanged:
                self.on_CIvalues_editingFinished()
            fct = self.fct
            res = self.res
            model = self.param_model
            xdata = model.valuesX
            ydata = model.valuesY
            p0 = model.parm_values
            parm_names = model.parm_names
            eval_points = None
            fixed = tuple(find(array(model.fixed) > 0))
            if self.interpolate.isChecked():
                if self.autoScale.isChecked():
                    xmin = xdata.min()
                    xmax = xdata.max()
                else:
                    xmin = float(self.xMin.text())
                    xmax = float(self.xMax.text())
                eval_points = arange(xmin, xmax, (xmax - xmin) / 1024)
            CImethod = None
            CImethodName = user_text(self.CImethod.currentText())
            if CImethodName == u"Bootstrapping":
                CImethod = bootstrap.bootstrap_regression
            elif CImethodName == u"Residual resampling":
                CImethod = bootstrap.bootstrap_residuals
            outfile = self.output
            CI = ()
            result = None
            loc = str(self.legendLocation.currentText())
            fct_desc = "$%s$" % (fct.description,)
            try:
                cf_args = (p0, fct)
                cf_kwrds = {"residuals": res.__call__, "maxfev": 10000, "fix_params": fixed,
                            "Dfun": fct.Dfun, "Dres": res.Dfun, "col_deriv": 1}
                if self.CI is not None:
                    CI = self.CI[1]
                    bs = bootstrap.bootstrap(CurveFitting, xdata, ydata, CI,
                                             shuffle_method=CImethod,
                                             shuffle_kwrds={"add_residual": res.invert,
                                                            "fit": CurveFitting},
                                             extra_attrs=('popt',), eval_points=eval_points,
                                             fit_args=cf_args, fit_kwrds=cf_kwrds)
                    result = plot_fit.fit_evaluation(bs.y_fit, xdata, ydata,
                                                     eval_points=eval_points, xname=self.fieldX,
                                                     yname=self.fieldY, fct_desc=fct_desc,
                                                     param_names=parm_names, res_name=res.name,
                                                     CI=CI, CIresults=bs)
                else:
                    fit = CurveFitting(xdata, ydata, *cf_args, **cf_kwrds)
                    result = plot_fit.fit_evaluation(fit, xdata, ydata, eval_points=eval_points,
                                                     xname=self.fieldX, yname=self.fieldY,
                                                     fct_desc=fct_desc, param_names=parm_names,
                                                     res_name=res.name)
            except Exception as ex:
                traceback.print_exc()
                QMessageBox.critical(self, "Error during Parameters Estimation",
                                     "{1} exception: {2}".format(type(ex).__name__, ex.message))
                return
            plot_fit.plot1d(result, loc=loc)
            if self.writeResult and outfile:
                #print("output to file '%s'" % (outfile,))
                plot_fit.write1d(outfile, result, res.description, CImethodName)
            #else:
                #print("self.writeResult = %s\noutfile='%s'" % (self.writeResult, outfile))


def main():
    wnd = QtFitDlg()
    wnd.show()
    wnd.raise_()
    return wnd

if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    matplotlib.interactive(True)
    wnd = main()
    app.exec_()



"""
:Author: Pierre Barbier de Reuille <pierre.barbierdereuille@gmail.com>

This module contains a set of methods to compute univariate KDEs. See the objects in the :py:mod:`pyqt_fit.kde` module 
for more details on these methods.
"""

import numpy as np
from scipy import fftpack
from .compat import irange

def generate_grid(kde, N=None, cut=None):
    r"""
    Helper method returning a regular grid on the domain of the KDE.

    :param KDE1D kde: Object describing the KDE computation
    :param int N: Number of points in the grid
    :param float cut: for unbounded domains, how far past the maximum should the grid extend to, in term of KDE
        bandwidth

    :return: A vector of N regularly spaced points
    """
    if N is None:
        N = 2 ** 10
    if cut is None:
        cut = 3
    if kde.lower == -np.inf:
        lower = np.min(kde.xdata) - cut * kde.bandwidth
    else:
        lower = kde.lower
    if kde.upper == np.inf:
        upper = np.max(kde.xdata) + cut * kde.bandwidth
    else:
        kde.upper
    return np.r_[lower:upper:N * 1j]

class KDE1DMethod(object):
    """
    Base class providing a default grid method and a default method for unbounded evaluation.
    """

    @staticmethod
    def unbounded(kde, points, output):
        """
        Method to use if there is, effectively, no bounds
        """
        xdata = kde.xdata
        points = np.atleast_1d(points)[:, np.newaxis]

        bw = kde.bandwidth * kde.lambdas

        z = (points - xdata) / bw

        kernel = kde.kernel

        terms = kernel(z)

        terms *= kde.weights / bw

        output = terms.sum(axis=1, out=output)
        output /= kde.total_weights

        return output

    __call__ = unbounded

    def grid(self, kde, N=None, cut=None):
        """
        Evaluate the method on a grid spanning the whole domain of the KDE and containing N points.

        :param KDE1D kde: KDE object
        :param int N: Number of points of the grid
        :param float cut: Cutting points for the unbounded domain (see :py:func:`generate_grid`)

        :returns: A tuple with the grid points and the estimated values on these points
        """
        g = generate_grid(kde, N, cut)
        return g, self(g)

    def __str__(self):
        """
        Return the name of the method
        """
        return self.name

class RenormalizationMethod(KDE1DMethod):
    r"""
    This method consists in using the normal kernel method, but renormalize
    to only take into account the part of the kernel within the domain of
    the density [1]_.

    The kernel is then replaced with:

    .. math::

        \hat{K}(x;X,h,L,U) \triangleq \frac{1}{a_0\left(\frac{L-x}{h},
        \frac{U-x}{h}\right)} K\left(\frac{x-X}{h}\right)

    .. [1] Jones, M. C. 1993. Simple boundary correction for kernel density
        estimation. Statistics and Computing 3: 135--146.
    """

    name = 'renormalization'

    @staticmethod
    def __call__(kde, points, output=None):
        if not kde.bounded:
            return KDE1DMethod.unbounded(kde, points, output)

        xdata = kde.xdata
        points = np.atleast_1d(points)[:, np.newaxis]

        bw = kde.bandwidth * kde.lambdas

        l = (kde.lower - points) / bw
        u = (kde.upper - points) / bw
        z = (points - xdata) / bw

        kernel = kde.kernel

        a1 = (kernel.cdf(u) - kernel.cdf(l))

        terms = kernel(z) * ((kde.weights / bw) / a1)

        output = terms.sum(axis=1, out=output)
        output /= kde.total_weights

        return output

renormalization = RenormalizationMethod()

class ReflectionMethod(KDE1DMethod):
    r"""
    This method consist in simulating the reflection of the data left and
    right of the boundaries. If one of the boundary is infinite, then the
    data is not reflected in that direction. To this purpose, the kernel is
    replaced with:

    .. math::

        \hat{K}(x; X, h, L, U) = K\left(\frac{x-X}{h}\right)
        + K\left(\frac{x+X-2L}{h}\right)
        + K\left(\frac{x+X-2U}{h}\right)

    When computing grids, if the bandwidth is constant, the result is
    computing using CDT.
    """

    name = 'reflection'

    @staticmethod
    def __call__(kde, points, output=None):
        if not kde.bounded:
            return KDE1DMethod.unbounded(kde, points, output)

        xdata = kde.xdata
        points = np.atleast_1d(points)[:, np.newaxis]

        # Make sure points are between the bounds, with reflection if needed
        if any(points < kde.lower) or any(points > kde.upper):
            span = kde.upper - kde.lower
            points = points - (kde.lower + span)
            points %= 2*span
            points -= kde.lower + span
            points = np.abs(points)

        bw = kde.bandwidth * kde.lambdas

        z = (points - xdata) / bw
        z1 = (points + xdata) / bw
        L = kde.lower
        U = kde.upper

        kernel = kde.kernel

        terms = kernel(z)

        if L > -np.inf:
            terms += kernel(z1 - (2 * L / bw))

        if U < np.inf:
            terms += kernel(z1 - (2 * U / bw))

        terms *= kde.weights / bw
        output = terms.sum(axis=1, out=output)
        output /= kde.total_weights

        return output

    @staticmethod
    def grid(kde, N=None, cut=None):
        """
        DCT-based estimation of KDE estimation, i.e. with reflection boundary
        conditions. This works only for fixed bandwidth (i.e. lambdas = 1) and
        gaussian kernel.

        For open domains, the grid is taken with 3 times the bandwidth as extra
        space to remove the boundary problems.
        """
        if kde.lambdas.shape:
            return KDE1DMethod.grid(kde, N, cut)

        bw = kde.bandwidth * kde.lambdas
        data = kde.xdata
        if N is None:
            N = 2 ** 14

        if kde.lower == -np.inf:
            lower = np.min(data) - 3 * kde.bandwidth
        else:
            lower = kde.lower
        if kde.upper == np.inf:
            upper = np.max(data) + 3 * kde.bandwidth
        else:
            upper = kde.upper

        R = upper - lower

        # Histogram the data to get a crude first approximation of the density
        weights = kde.weights
        if not weights.shape:
            weights = None
        DataHist, bins = np.histogram(data, bins=N, range=(lower, upper),
                                      weights=weights)
        DataHist = DataHist / kde.total_weights
        DCTData = fftpack.dct(DataHist, norm=None)

        if hasattr(kde.kernel, 'dct'):
            t_star = bw / R
            gp = np.arange(N) * np.pi * t_star
            smth = kde.kernel.dct(gp)
        else:
            gp = (np.arange(N) + 0.5) * R / N
            smth = fftpack.dct(kde.kernel(gp / bw) * (gp[1] - gp[0]) / bw)

        # Smooth the DCTransformed data using t_star
        SmDCTData = DCTData * smth
        # Inverse DCT to get density
        density = fftpack.idct(SmDCTData, norm=None) / (2 * R)
        mesh = np.array([(bins[i] + bins[i + 1]) / 2 for i in irange(N)])

        return mesh, density

reflection = ReflectionMethod()

class LinearCombinationMethod(KDE1DMethod):
    r"""
    This method uses the linear combination correction published in [1]_.

    The estimation is done with a modified kernel given by:

    .. math::

        K_r(x;X,h,L,U) = \frac{a_2(l,u) - a_1(-u, -l) z}{a_2(l,u)a_0(l,u)
        - a_1(-u,-l)^2} K(z)

        z = \frac{x-X}{h} \qquad l = \frac{L-x}{h} \qquad u = \frac{U-x}{h}

    .. [1] Jones, M. C. 1993. Simple boundary correction for kernel density
        estimation. Statistics and Computing 3: 135--146.
    """

    name = 'linear combination'

    @staticmethod
    def __call__(kde, points, output=None):
        if not kde.bounded:
            return KDE1DMethod.unbounded(kde, points, output)

        xdata = kde.xdata
        points = np.atleast_1d(points)[:, np.newaxis]

        bw = kde.bandwidth * kde.lambdas

        l = (kde.lower - points) / bw
        u = (kde.upper - points) / bw
        z = (points - xdata) / bw

        kernel = kde.kernel

        a0 = kernel.cdf(u) - kernel.cdf(l)
        a1 = kernel.pm1(-l) - kernel.pm1(-u)
        a2 = kernel.pm2(u) - kernel.pm2(l)

        denom = a2 * a0 - a1 * a1
        upper = a2 - a1 * z

        upper /= denom
        upper *= (kde.weights / bw) * kernel(z)

        output = upper.sum(axis=1, out=output)
        output /= kde.total_weights

        return output

linear_combination = LinearCombinationMethod()

class CyclicMethod(KDE1DMethod):
    r"""
    This method assumes cyclic boundary conditions and works only for
    closed boundaries.

    The estimation is done with a modified kernel given by:

    .. math::

        \hat{K}(x; X, h, L, U) = K\left(\frac{x-X}{h}\right)
        + K\left(\frac{x-X-(U-L)}{h}\right)
        + K\left(\frac{x-X+(U-L)}{h}\right)

    When computing grids, if the bandwidth is constant, the result is
    computing using FFT.
    """

    name = 'cyclic'

    @staticmethod
    def __call__(kde, points, output=None):
        if not kde.closed:
            raise ValueError("Cyclic boundary conditions can only be used with "
                             "closed domains.")

        xdata = kde.xdata
        points = np.atleast_1d(points)[:, np.newaxis]

        # Make sure points are between the bounds
        if any(points < kde.lower) or any(points > kde.upper):
            points = points - kde.lower
            points %= kde.upper - kde.lower
            points += kde.lower

        bw = kde.bandwidth * kde.lambdas

        z = (points - xdata) / bw
        L = kde.lower
        U = kde.upper

        span = U - L

        kernel = kde.kernel

        terms = kernel(z)
        terms += kernel(z - (span / bw))
        terms += kernel(z + (span / bw))

        terms *= kde.weights / bw
        output = terms.sum(axis=1, out=output)
        output /= kde.total_weights

        return output

    def grid(kde, N=None, cut=None):
        """
        FFT-based estimation of KDE estimation, i.e. with cyclic boundary
        conditions. This works only for closed domains, fixed bandwidth
        (i.e. lambdas = 1) and gaussian kernel.
        """
        if kde.lambdas.shape:
            return KDE1DMethod.grid(kde, N, cut)
        if not kde.closed:
            raise ValueError("Error, cyclic boundary conditions require "
                             "a closed domain.")
        bw = kde.bandwidth * kde.lambdas
        data = kde.xdata
        if N is None:
            N = 2 ** 14
        lower = kde.lower
        upper = kde.upper
        R = upper - lower
        dN = 1 / N
        mesh = np.r_[lower:upper + dN:(N + 2) * 1j]
        weights = kde.weights
        if not weights.shape:
            weights = None
        DataHist, bin_edges = np.histogram(data, bins=mesh - dN / 2,
                                           weights=weights)
        DataHist[0] += DataHist[-1]
        DataHist = DataHist / kde.total_weights
        FFTData = fftpack.fft(DataHist[:-1])
        if hasattr(kde.kernel, 'fft'):
            t_star = (2 * bw / R)
            gp = np.roll((np.arange(N) - N / 2) * np.pi * t_star, N // 2)
            smth = kde.kernel.fft(gp)
        else:
            gp = np.roll((np.arange(N) - N / 2) * R / N, N // 2)
            smth = fftpack.fft(kde.kernel(gp / bw) * (gp[1] - gp[0]) / bw)
        SmoothFFTData = FFTData * smth
        density = fftpack.ifft(SmoothFFTData) / (mesh[1] - mesh[0])
        return mesh[:-2], density.real

cycle = CyclicMethod()



"""
:Author: Pierre Barbier de Reuille <pierre.barbierdereuille@gmail.com>

Module implementing kernel-based estimation of density of probability.
"""

from __future__ import division, absolute_import, print_function
import numpy as np
from .kernels import normal_kernel1d
from .utils import namedtuple, large_float
from . import kde_methods
from .kde_bandwidth import variance_bandwidth, silverman_covariance, scotts_covariance, botev_bandwidth


class KDE1D(object):
    r"""
    Perform a kernel based density estimation in 1D, possibly on a bounded
    domain :math:`[L,U]`.

    :param ndarray data: 1D array with the data points

    Any other named argument will be equivalent to setting the property
    after the fact. For example::

        >>> xs = [1,2,3]
        >>> k = KDE1D(xs, lower=0)

    will be equivalent to::

        >>> k = KDE1D(xs)
        >>> k.lower = 0

    The method rely on an estimator of kernel density given by:

    .. math::

        f(x) \triangleq \frac{1}{hW} \sum_{i=1}^n \frac{w_i}{\lambda_i}
        K\left(\frac{X-x}{h\lambda_i}\right)

        W = \sum_{i=1}^n w_i

    where :math:`h` is the bandwidth of the kernel (:py:attr:`bandwidth`),
    and :math:`K` is the kernel used for the density estimation
    (:py:attr:`kernel`), :math:`w_i` are the weights of the data points
    (:py:attr:`weights`) and :math:`\lambda_i` are the adaptation factor
    of the kernel width (:py:attr:`lambdas`). :math:`K` should be a function
    such that:

    .. math::

        \begin{array}{rcl}
        \int_\mathbb{R} K(z) &=& 1 \\
        \int_\mathbb{R} zK(z)dz &=& 0 \\
        \int_\mathbb{R} z^2K(z) dz &<& \infty \quad (\approx 1)
        \end{array}

    Which translates into, the function should be of sum 1 (i.e.
    a valid density of probability), of average 0 (i.e. centered) and of finite
    variance. It is even recommanded that the variance is close to 1 to give
    a uniform meaning to the bandwidth.

    If the domain of the density estimation is bounded to the interval
    :math:`[L,U]` (i.e. from :py:attr:`lower` to :py:attr:`upper`), the density
    is then estimated with:

    .. math::

        f(x) \triangleq \frac{1}{hW} \sum_{i=1}^n \frac{w_i}{\lambda_i}
        \hat{K}(x;X,\lambda_i h,L,U)

    Where :math:`\hat{K}` is a modified kernel that depends on the exact method
    used.

    To express the various methods, we will refer to the following functions:

    .. math::

        a_0(l,u) = \int_l^u K(z) dz

        a_1(l,u) = \int_l^u zK(z) dz

        a_2(l,u) = \int_l^u z^2K(z) dz


    The default methods are implemented in the `kde_methods` module.
    """

    def __init__(self, xdata, **kwords):
        self._xdata = None
        self._upper = np.inf
        self._lower = -np.inf
        self._kernel = normal_kernel1d()

        self._bw_fct = None
        self._bw = None
        self._cov_fct = None
        self._covariance = None
        self._method = None

        self.weights = 1.
        self.lambdas = 1.

        self._initialized = False

        for n in kwords:
            setattr(self, n, kwords[n])

        self.xdata = np.atleast_1d(xdata)

        has_bw = (self._bw is not None or self._bw_fct is not None or
                  self._covariance is not None or self._cov_fct is not None)
        if not has_bw:
            self.covariance = scotts_covariance

        if self._method is None:
            self.method = kde_methods.renormalization

        self._initialized = True
        self.update_bandwidth()

    @property
    def initialized(self):
        return self._initialized

    def copy(self):
        """
        Shallow copy of the KDE object
        """
        res = KDE1D.__new__(KDE1D)
        # Copy private members: start with a single '_'
        for m in self.__dict__:
            if len(m) > 1 and m[0] == '_' and m[1] != '_':
                setattr(res, m, getattr(self, m))
        return res

    def update_bandwidth(self):
        """
        Re-compute the bandwidth if it was specified as a function.
        """
        if self._xdata is None or not self.initialized:
            return
        if self._bw_fct:
            _bw = float(self._bw_fct(self._xdata, model=self))
            _cov = _bw * _bw
        elif self._cov_fct:
            _cov = float(self._cov_fct(self._xdata, model=self))
            _bw = np.sqrt(_cov)
        else:
            return
        self._covariance = _cov
        self._bw = _bw

    @property
    def xdata(self):
        return self._xdata

    @xdata.setter
    def xdata(self, xs):
        self._xdata = np.atleast_1d(xs)
        self.update_bandwidth()

    @property
    def kernel(self):
        r"""
        Kernel object. Should provide the following methods:

        ``kernel.pdf(xs)``
            Density of the kernel, denoted :math:`K(x)`

        ``kernel.cdf(z)``
            Cumulative density of probability, that is
            :math:`F^K(z) = \int_{-\infty}^z K(x) dx`

        ``kernel.pm1(z)``
            First partial moment, defined by
            :math:`\mathcal{M}^K_1(z) = \int_{-\infty}^z xK(x)dx`

        ``kernel.pm2(z)``
            Second partial moment, defined by
            :math:`\mathcal{M}^K_2(z) = \int_{-\infty}^z x^2K(x)dx`

        ``kernel.fft(z)``
            FFT of the kernel on the points of ``z``. The points will always be
            provided as a grid with :math:`2^n` points, representing the whole
            frequency range to be explored. For convenience, the second half of
            the points will be provided as negative values.

        ``kernel.dct(z)``
            DCT of the kernel on the points of ``z``. The points will always be
            provided as a grid with :math:`2^n` points, representing the whole
            frequency range to be explored.

        By default, the kernel is an instance of
        :py:class:`kernels.normal_kernel1d`
        """
        return self._kernel

    @kernel.setter
    def kernel(self, val):
        self._kernel = val

    @property
    def lower(self):
        r"""
        Lower bound of the density domain. If deleted, becomes set to
        :math:`-\infty`
        """
        return self._lower

    @lower.setter
    def lower(self, val):
        self._lower = float(val)

    @lower.deleter
    def lower(self):
        self._lower = -np.inf

    @property
    def upper(self):
        r"""
        Upper bound of the density domain. If deleted, becomes set to
        :math:`\infty`
        """
        return self._upper

    @upper.setter
    def upper(self, val):
        self._upper = float(val)

    @upper.deleter
    def upper(self):
        self._upper = np.inf

    @property
    def weights(self):
        """
        Weigths associated to each data point. It can be either a single value,
        or an array with a value per data point. If a single value is provided,
        the weights will always be set to 1.
        """
        return self._weights

    @weights.setter
    def weights(self, ws):
        try:
            ws = float(ws)
            self._weights = np.asarray(1.)
        except TypeError:
            ws = np.array(ws, dtype=float)
            self._weights = ws
        self._total_weights = None

    @weights.deleter
    def weights(self):
        self._weights = np.asarray(1.)
        self._total_weights = None

    @property
    def total_weights(self):
        if self._total_weights is None:
            if self._weights.shape:
                assert self._weights.shape == self._xdata.shape, \
                    "There must be as many weigths as data points"
                self._total_weights = sum(self._weights)
            else:
                self._total_weights = len(self._xdata)
        return self._total_weights

    @property
    def lambdas(self):
        """
        Scaling of the bandwidth, per data point. It can be either a single
        value or an array with one value per data point.

        When deleted, the lamndas are reset to 1.
        """
        return self._lambdas

    @lambdas.setter
    def lambdas(self, ls):
        try:
            self._lambdas = np.asarray(float(ls))
        except TypeError:
            ls = np.array(ls, dtype=float)
            self._lambdas = ls

    @lambdas.deleter
    def lambdas(self):
        self._lambdas = np.asarray(1.)

    @property
    def bandwidth(self):
        """
        Bandwidth of the kernel.
        Can be set either as a fixed value or using a bandwidth calculator,
        that is a function of signature ``w(xdata)`` that returns a single
        value.

        .. note::

            A ndarray with a single value will be converted to a floating point
            value.
        """
        return self._bw

    @bandwidth.setter
    def bandwidth(self, bw):
        self._bw_fct = None
        self._cov_fct = None
        if callable(bw):
            self._bw_fct = bw
            self.update_bandwidth()
        else:
            bw = float(bw)
            self._bw = bw
            self._covariance = bw * bw

    @property
    def covariance(self):
        """
        Covariance of the gaussian kernel.
        Can be set either as a fixed value or using a bandwidth calculator,
        that is a function of signature ``w(xdata)`` that returns a single
        value.

        .. note::

            A ndarray with a single value will be converted to a floating point
            value.
        """
        return self._covariance

    @covariance.setter
    def covariance(self, cov):
        self._bw_fct = None
        self._cov_fct = None
        if callable(cov):
            self._cov_fct = cov
            self.update_bandwidth()
        else:
            cov = float(cov)
            self._covariance = cov
            self._bw = np.sqrt(cov)

    def evaluate(self, points, output=None):
        """
        Evaluate the kernel on the set of points ``points``
        """
        return self._method(self, points, output)

    def __call__(self, points, output=None):
        """
        This method is an alias for :py:meth:`BoundedKDE1D.evaluate`
        """
        return self.evaluate(points, output=output)

    @property
    def method(self):
        """
        Select the method to use. Available methods in the :py:mod:`pyqt_fit.kde_methods` sub-module.

        The method is an object that should provide the following:

        ``method(kde, points, output)``
            Evaluate the KDE defined by the ``kde`` object on the ``points``. If ``output`` is provided, it should have 
            the right shape and the result should be written in it.

        ``method.grid(kde, N, cut)``
            Evaluate the KDE defined by the ``kde`` object on a grid. See :py:fct:`pyqt_fit.kde_methods.generate_grid` 
            for a detailed explanation on how the grid is computed.

        ``method.name``
            Return a user-readable name for the method

        ``str(method)``
            Should return the method's name
        """
        return self._method

    @method.setter
    def method(self, m):
        self._method = m

    @property
    def closed(self):
        """
        Returns true if the density domain is closed (i.e. lower and upper
        are both finite)
        """
        return self.lower > -np.inf and self.upper < np.inf

    @property
    def bounded(self):
        """
        Returns true if the density domain is actually bounded
        """
        return self.lower > -np.inf or self.upper < np.inf

    def grid(self, N=None, cut=None):
        """
        Evaluate the density on a grid of N points spanning the whole dataset.

        :returns: a tuple with the mesh on which the density is evaluated and
        the density itself
        """
        return self._method.grid(self, N, cut)

Transform = namedtuple('Tranform', ['__call__', 'inv', 'Dinv'])

LogTransform = Transform(np.log, np.exp, np.exp)
ExpTransform = Transform(np.exp, np.log, lambda x: 1 / x)


def transform_distribution(xs, ys, Dfct, output=None):
    """
    Transform a distribution into another one by a change a variable.

    Given a random variable :math:`X` of distribution :math:`f_X`, the random
    variable :math:`Y = g(X)` has a distribution :math:`f_Y` given by:

    .. math::

        f_Y(y) = \left| \frac{1}{g'(g^{-1}(y))} \right| \cdot f_X(g^{-1}(y))

    """
    return np.multiply(np.abs(1 / Dfct(xs)), ys, output)


def create_transform(obj, inv=None, Dinv=None):
    if isinstance(obj, Transform):
        return obj
    fct = obj.__call__
    if inv is None:
        if not hasattr(obj, 'inv'):
            raise AttributeError("Error, transform object must have a 'inv' "
                                 "attribute or you must specify 'inv'")
        inv = obj.inv if hasattr(obj, 'inv') else inv
    if Dinv is None:
        if hasattr(obj, Dinv):
            Dinv = obj.Dinv
        else:
            def Dinv(x):
                x = np.asfarray(x)
                dx = x * 1e-9
                dx[x == 0] = np.min(dx[x != 0])
                return (inv(x + dx) - inv(x - dx)) / (2 * dx)
    return Transform(fct, inv, Dinv)


class TransformKDE(object):
    r"""
    Compute the Kernel Density Estimate of a dataset, transforming it first to
    a domain where distances are "more meaningful".

    Often, KDE is best estimated in a different domain. This object takes a
    KDE1D object (or one compatible), and a transformation function.

    Given a random variable :math:`X` of distribution :math:`f_X`, the random
    variable :math:`Y = g(X)` has a distribution :math:`f_Y` given by:

    .. math::

        f_Y(y) = \left| \frac{1}{g'(g^{-1}(y))} \right| \cdot f_X(g^{-1}(y))

    In our term, :math:`Y` is the random variable the user is interested in,
    and :math:`X` the random variable we can estimate using the KDE. In this
    case, :math:`g` is the transform from :math:`Y` to :math:`X`.

    So to estimate the distribution on a set of points given in :math:`x`, we
    need a total of three functions:

        - Direct function: transform from the original space to the one in
          which the KDE will be perform (i.e. :math:`g^{-1}: y \mapsto x`)
        - Invert function: transform from the KDE space to the original one
          (i.e. :math:`g: x \mapsto y`)
        - Derivative of the invert function

    If the derivative is not provided, it will be estimated numerically.

    :param kde: KDE evaluation object
    :param trans: Either a simple function, or a function object with
        attributes `inv` and `Dinv` to use in case they are not provided
        as arguments.
    :param inv: Invert of the function. If not provided, `trans` must have
        it as attribute.
    :param Dinv: Derivative of the invert function.

    Any unknown member is forwarded to the underlying KDE object.
    """
    def __init__(self, kde, trans, inv=None, Dinv=None):
        d = self.__dict__
        trans = create_transform(trans, inv, Dinv)
        d['trans'] = trans
        d['kde'] = kde.copy()
        self.kde.xdata = trans(kde.xdata)
        self.kde.lower = trans(kde.lower)
        self.kde.upper = trans(kde.upper)
        self.kde.update_bandwidth()

    def copy(self):
        """
        Creates a shallow copy of the TransformKDE object
        """
        res = TransformKDE.__new__(TransformKDE)
        d = res.__dict__
        d['trans'] = self.trans
        d['kde'] = self.kde
        return res

    def evaluate(self, points, output=None):
        """
        Evaluate the KDE on a set of points
        """
        trans = self.trans
        pts = trans(points)
        output = self.kde(pts, output)
        return transform_distribution(pts, output, trans.Dinv, output)

    def __call__(self, points, output=None):
        """
        Evaluate the KDE on a set of points
        """
        return self.evaluate(points, output)

    def grid(self, N=None):
        """
        Evaluate the KDE on a grid of points with N points.

        The grid is regular *in the transformed domain*, so as to use FFT or
        CDT methods when applicable.
        """
        xs, ys = self.kde.grid(N)
        trans = self.trans
        return trans.inv(xs), transform_distribution(xs, ys, trans.Dinv)

    def __getattr__(self, name):
        return getattr(self.kde, name)

    def __setattr__(self, name, value):
        return setattr(self.kde, name, value)



from __future__ import division
from numpy import log, exp, newaxis


class Standard(object):
    name = "Standard"
    description = "y_1 - y_0"

    @staticmethod
    def __call__(y1, y0):
        return y1 - y0

    @staticmethod
    def invert(y, res):
        """
        Add the residual to the value
        """
        return y + res

    @staticmethod
    def Dfun(y1, y0, dy):
        """
        J(y1-y0) = J(y1)-J(y0) = -J(y0)
        where J is the jacobian
        """
        return -dy


class LogResiduals(object):
    name = "Difference of the logs"
    description = "log(y1/y0)"

    @staticmethod
    def __call__(y1, y0):
        return log(y1 / y0)

    @staticmethod
    def Dfun(y1, y0, dy):
        """
        J(log(y1/y0)) = -J(y0)/y0
        where J is the jacobian and division is element-wise (per row)
        """
        return -dy / y0[newaxis, :]

    @staticmethod
    def invert(y, res):
        """
        Multiply the value by the exponential of the residual
        """
        return y * exp(res)
